from agora_rest_client.core import request
from agora_rest_client.services.cloud_recording.v1 import api_start
from agora_rest_client.services.cloud_recording.v1.api import AvFileType
from agora_rest_client.services.cloud_recording.v1.api import Mode

class StorageConfig(api_start.StorageConfig):
    pass

class RecordingFileConfig(api_start.RecordingFileConfig):
    pass

class ExtensionServiceConfig(api_start.ExtensionServiceConfig):
    pass

class ExtensionServices(api_start.ExtensionServices):
    pass

class ServiceParam(api_start.ServiceParam):
    pass

class ClientRequest(request.RequestObject):
    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.ClientRequest.storageConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.web_recording.api_start.StorageConfig`
    """
    storageConfig = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.ClientRequest.recordingFileConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.web_recording.api_start.RecordingFileConfig`
    """
    recordingFileConfig = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.ClientRequest.extensionServiceConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.web_recording.api_start.ExtensionServiceConfig`
    """
    extensionServiceConfig = None

class RequestPathParamsApiStart(api_start.RequestPathParamsApiStart):
    pass

class RequestBodyApiStart(api_start.RequestBodyApiStart):
    pass

class ResponseApiStart(api_start.ResponseApiStart):
    pass

def web_recording_start(client, resource_id, cname, uid, storage_config, extension_service_config, trace_id=None):
    """
    Web recording start
    开始云端录制

    :type client: object
    :param client: WebRecordingClient object

    :type resource_id: str
    :param resource_id: resource id
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.RequestPathParamsApiStart.resource_id`

    :type cname: str
    :param cname: cname
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.RequestBodyApiStart.cname`

    :type uid: str
    :param uid: uid
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.RequestBodyApiStart.uid`

    :type storage_config: object
    :param storage_config: storage config
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.StorageConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.web_recording.api_start.StorageConfig`

    :type extension_service_config: object
    :param extension_service_config: extension service config
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.ExtensionServiceConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.web_recording.api_start.ExtensionServiceConfig`

    :type trace_id: string
    :param trace_id: trace id

    :return: response object ResponseApiStart
    """
    request_path_params_obj = RequestPathParamsApiStart(
        mode=Mode.WEB.value,
        resource_id=resource_id
    )

    request_body_obj = RequestBodyApiStart(
        cname=cname,
        uid=uid,
        clientRequest=ClientRequest(
            storageConfig=storage_config,
            recordingFileConfig=RecordingFileConfig(
                avFileType=[AvFileType.HLS.value, AvFileType.MP4.value]
            ),
            extensionServiceConfig=extension_service_config
        )
    )

    return api_start.api_start(client, request_path_params_obj=request_path_params_obj, request_body_obj=request_body_obj,
                               response_obj=ResponseApiStart, trace_id=trace_id)



from agora_rest_client.core import request
from agora_rest_client.services.cloud_recording.v1 import api_start
from agora_rest_client.services.cloud_recording.v1.api import AvFileType
from agora_rest_client.services.cloud_recording.v1.api import Mode
from agora_rest_client.services.cloud_recording.v1.api import SnapshotType

class StorageConfig(api_start.StorageConfig):
    pass

class RecordingConfig(api_start.RecordingConfig):
    pass

class RecordingFileConfig(api_start.RecordingFileConfig):
    pass

class SnapshotConfig(api_start.SnapshotConfig):
    pass

class AppsCollection(api_start.AppsCollection):
    pass

class TranscodeOptions(api_start.TranscodeOptions):
    pass

class ClientRequest(request.RequestObject):
    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.ClientRequest.token`
    """
    token = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.ClientRequest.storageConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.individual_recording.api_start.StorageConfig`
    """
    storageConfig = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.ClientRequest.recordingConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.individual_recording.api_start.RecordingConfig`
    """
    recordingConfig = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.ClientRequest.recordingFileConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.individual_recording.api_start.RecordingFileConfig`
    """
    recordingFileConfig = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.ClientRequest.snapshotConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.individual_recording.api_start.SnapshotConfig`
    """
    snapshotConfig = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.ClientRequest.appsCollection`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.individual_recording.api_start.AppsCollection`
    """
    appsCollection = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.ClientRequest.transcodeOptions`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.individual_recording.api_start.TranscodeOptions`
    """
    transcodeOptions = None

class RequestPathParamsApiStart(api_start.RequestPathParamsApiStart):
    pass

class RequestBodyApiStart(api_start.RequestBodyApiStart):
    pass

class ResponseApiStart(api_start.ResponseApiStart):
    pass

def individual_recording_start(client, resource_id, cname, uid, token, storage_config, recording_config=None,
                               snapshot_type=SnapshotType.SNAPSHOT_AND_RECORDING.value, snapshot_config=None, apps_collection=None, transcode_options=None, trace_id=None):
    """
    Individual recording start
    开始云端录制

    :type client: object
    :param client: IndividualRecordingClientgClient object

    :type resource_id: str
    :param resource_id: resource id
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.RequestPathParamsApiStart.resource_id`

    :type cname: str
    :param cname: cname
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.RequestBodyApiStart.cname`

    :type uid: str
    :param uid: uid
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.RequestBodyApiStart.uid`

    :type token: str
    :param token: token
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.ClientRequest.token`

    :type storage_config: object
    :param storage_config: storage config
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.StorageConfig`

    :type recording_config: object
    :param recording_config: recording config
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.RecordingConfig`

    :type snapshot_type: int
    :param snapshot_type: 视频截图类型
    :refer: `agora_rest_client.services.cloud_recording.v1.api.SnapshotType`
    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.SnapshotType`

    :type snapshot_config: object
    :param snapshot_config: snapshot config
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.SnapshotConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.individual_recording.api_start.SnapshotConfig`

    :type apps_collection: object
    :param apps_collection: apps collection
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.AppsCollection`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.individual_recording.api_start.AppsCollection`

    :type transcode_options: object
    :param transcode_options: transcode options
    :refer: `agora_rest_client.services.cloud_recording.v1.api_start.TranscodeOptions`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.individual_recording.api_start.TranscodeOptions`

    :type trace_id: string
    :param trace_id: trace id

    :return: response object ResponseApiStart
    """
    request_path_params_obj = RequestPathParamsApiStart(
        mode=Mode.INDIVIDUAL.value,
        resource_id=resource_id
    )

    request_body_obj = RequestBodyApiStart(
        cname=cname,
        uid=uid,
        clientRequest=ClientRequest(
            token=token,
            storageConfig=storage_config,
        )
    )

    if recording_config is not None:
        request_body_obj.clientRequest.recordingConfig = recording_config

    if snapshot_type is SnapshotType.SNAPSHOT_AND_RECORDING.value:
        request_body_obj.clientRequest.recordingFileConfig = RecordingFileConfig(
            avFileType=[AvFileType.HLS.value]
        )

    if snapshot_config is not None:
        request_body_obj.clientRequest.snapshotConfig = snapshot_config

    if apps_collection is not None:
        request_body_obj.clientRequest.appsCollection = apps_collection

    if transcode_options is not None:
        request_body_obj.clientRequest.transcodeOptions = transcode_options

    return api_start.api_start(client, request_path_params_obj=request_path_params_obj, request_body_obj=request_body_obj,
                               response_obj=ResponseApiStart, trace_id=trace_id)



from agora_rest_client.core import request
from agora_rest_client.services.cloud_recording.v1 import api_update
from agora_rest_client.services.cloud_recording.v1.api import Mode

class WebRecordingConfig(api_update.WebRecordingConfig):
    pass

class RtmpPublishConfig(api_update.RtmpPublishConfig):
    pass

class ClientRequest(request.RequestObject):
    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update.ClientRequest.webRecordingConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.web_recording.api_update.WebRecordingConfig`
    """
    webRecordingConfig = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update.ClientRequest.rtmpPublishConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.web_recording.api_update.RtmpPublishConfig`
    """
    rtmpPublishConfig = None

class RequestPathParamsApiUpdate(api_update.RequestPathParamsApiUpdate):
    pass

class RequestBodyApiUpdate(api_update.RequestBodyApiUpdate):
    pass

class ResponseApiUpdate(api_update.ResponseApiUpdate):
    pass

def web_recording_update(client, resource_id, sid, cname, uid, web_recording_config=None, rtmp_publish_config=None, trace_id=None):
    """
    Web recording update
    更新云端录制设置

    :type client: object
    :param client: WebRecordingClient object

    :type resource_id: str
    :param resource_id: resource id
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update.RequestPathParamsApiUpdate.resource_id`

    :type sid: str
    :param sid: sid
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update.RequestPathParamsApiUpdate.sid`

    :type cname: str
    :param cname: cname
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update.RequestBodyApiUpdate.cname`

    :type uid: str
    :param uid: uid
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update.RequestBodyApiUpdate.uid`

    :type web_recording_config: object
    :param web_recording_config: web recording config
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update.WebRecordingConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.web_recording.api_update.WebRecordingConfig`

    :type rtmp_publish_config: object
    :param rtmp_publish_config: rtmp publish config
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update.RtmpPublishConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.web_recording.api_update.RtmpPublishConfig`

    :type trace_id: string
    :param trace_id: trace id

    :return: response object ResponseApiUpdate
    """
    request_path_params_obj = RequestPathParamsApiUpdate(
        mode=Mode.WEB.value,
        resource_id=resource_id,
        sid=sid
    )

    request_body_obj = RequestBodyApiUpdate(
        cname=cname,
        uid=uid,
        clientRequest=ClientRequest()
    )

    if web_recording_config is not None:
        request_body_obj.clientRequest.webRecordingConfig = web_recording_config

    if rtmp_publish_config is not None:
        request_body_obj.clientRequest.rtmpPublishConfig = rtmp_publish_config

    return api_update.api_update(client, request_path_params_obj=request_path_params_obj, request_body_obj=request_body_obj,
                                 response_obj=ResponseApiUpdate, trace_id=trace_id)



from agora_rest_client.services.cloud_recording.v1 import api_stop
from agora_rest_client.services.cloud_recording.v1.api import Mode

class RequestPathParamsApiStop(api_stop.RequestPathParamsApiStop):
    pass

class RequestBodyApiStop(api_stop.RequestBodyApiStop):
    pass

class ResponseApiStop(api_stop.ResponseApiStop):
    pass

def mix_recording_stop(client, resource_id, sid, cname, uid, async_stop=False, trace_id=None):
    """
    Mix recording stop
    停止云端录制

    :type client: object
    :param client: MixRecordingClient object

    :type resource_id: str
    :param resource_id: resource id
    :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.RequestPathParamsApiStop.resource_id`

    :type sid: str
    :param sid: sid
    :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.RequestPathParamsApiStop.sid`

    :type cname: str
    :param cname: cname
    :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.RequestBodyApiStop.cname`

    :type uid: str
    :param uid: uid
    :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.RequestBodyApiStop.uid`

    :type async_stop: bool
    :param async_stop: async stop
    :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.ClientRequest.async_stop`

    :type trace_id: string
    :param trace_id: trace id

    :return: response object ResponseApiStop
    """
    request_path_params_obj = RequestPathParamsApiStop(
        mode=Mode.MIX.value,
        resource_id=resource_id,
        sid=sid
    )

    request_body_obj = RequestBodyApiStop(
        cname=cname,
        uid=uid,
        clientRequest=api_stop.ClientRequest(
            async_stop=async_stop
        )
    )

    return api_stop.api_stop(client, request_path_params_obj=request_path_params_obj, request_body_obj=request_body_obj,
                             response_obj=ResponseApiStop, trace_id=trace_id)



"""
错误码和响应状态码

官方文档: https://doc.shengwang.cn/api-ref/cloud-recording/restful/response-code

错误码
    当调用云端录制 RESTful API 响应码不为 200 时，响应 Body 中会包含一个错误码。

    2：参数不合法，请确保参数类型、大小写和取值范围正确，且必填的参数均已填写。

    7：录制已经在进行中 ，请勿用同一个 Resource ID 重复 start 请求。

    8：HTTP 请求头部字段错误，有以下几种情况：
        Content-type 错误，请确保 Content-type 为 application/json;charset=utf-8。
        请求 URL 中缺少 cloud_recording 字段。
        使用了错误的 HTTP 方法。
        请求包体不是合法的 JSON 格式。

    49：使用同一个 Resource ID 和录制 ID (sid) 重复 stop 请求。

    53：录制已经在进行中。当采用相同参数再次调用 acquire 获得新的 Resource ID，并用于 start 请求时，会发生该错误。如需发起多路录制，需要在 acquire 方法中填入不同的 UID。

    62：如果调用 acquire 请求时出现该错误，则表示你填入的 App ID 没有开通云端录制服务。如果调用 start 请求时出现该错误，则表示由于网络原因没有连接到录制服务，请重新发起请求。

    65：多为网络抖动引起。当调用 start 方法收到该错误码时，需要使用同一 Resource ID 再次调用 start。建议使用退避策略重试两次，如第一次等待 3 秒后重试、第二次等待 6 秒后重试。

    109：加入 RTC 频道的鉴权 Token 过期，你需要获取新的 Token。

    110：加入 RTC 频道的鉴权 Token 错误，你需要确认你是否获取了正确的 Token。

    432：请求参数错误。请求参数不合法，或请求中的 App ID，频道名或用户 ID 与 Resource ID 不匹配。

    433：Resource ID 过期。获得 Resource ID 后必须在 5 分钟内开始云端录制。请重新调用 acquire 获取新的 Resource ID。

    435：没有录制文件产生。频道内没有用户发流，无录制对象。

    501：录制服务正在退出。该错误可能在调用了 stop 方法后再调用 query 时发生。

    1001：Resource ID 解密失败。请重新调用 acquire 获取新的 Resource ID。

    1003：App ID 或者录制 ID（sid）与 Resource ID 不匹配。请确保在一个录制周期内 Resource ID、App ID 和录制 ID 一一对应。

    1013：频道名不合法。频道名必须为长度在 64 字节以内的字符串。以下为支持的字符集范围（共 89 个字符）：
        26 个小写英文字母 a-z
        26 个大写英文字母 A-Z
        10 个数字 0-9
        空格
        "!"、"#"、"$"、"%"、"&"、"("、")"、"+"、"-"、":"、";"、"<"、"="、"."、">"、"?"、"@"、"["、"]"、"^"、"_"、"、"、"|"、"~"、","

    1028：updateLayout 方法的请求包体中参数错误。

    "invalid appid"：无效的 App ID。请确保 App ID 填写正确。如果你已经确认 App ID 填写正确，但仍出现该错误，请联系技术支持。

    "no Route matched with those values"：该错误可能由 HTTP 方法填写错误导致，例如将 GET 方法填写为 POST；也可能由请求 URL 填写错误导致。

    "Invalid authentication credentials"：该错误可能由以下原因导致。如果你已经排除以下原因，但仍出现该错误，请联系联系技术支持。
        客户 ID 或客户密钥填写错误。
        App ID 没有开通云端录制服务。
        HTTP 请求头的认证信息有误，如 Authorization 字段的值 Basic <Authorization> 缺少 Basic。
        HTTP 请求头的格式不正确，如 Content-type 字段的值 application/json;charset=utf-8 大小写不正确或包含空格。


响应状态码
    状态码                     描述
    200                       请求成功。
    201                       录制已经在进行中 ，请勿用同一个 Resource ID 重复 start 请求。
    206                       整个录制过程中没有用户发流，或部分录制文件没有上传到第三方云存储，或录制进程还未结束。
    400                       请求的语法错误（如参数错误）。如果你填入的 App ID 没有开通云端录制服务，也会返回 400。
    401                       未经授权的（客户 ID/客户密钥匹配错误）。
    404                       服务器无法根据请求找到资源（网页）。
    500                       服务器内部错误，无法完成请求。
    504                       服务器内部错误。充当网关或代理的服务器未从远端服务器获取请求。


服务状态
    状态                           描述
    "serviceIdle"                 子模块服务未开始。
    "serviceStarted"              子模块服务已开始。
    "serviceReady"                子模块服务已就绪。
    "serviceInProgress"           子模块服务正在进行中。
    "serviceCompleted"            订阅内容已全部上传至扩展服务。
    "servicePartialCompleted"     订阅内容部分上传至扩展服务。
    "serviceValidationFailed"     扩展服务验证失败。例如 extensionServiceConfig 中 apiData 填写错误。
    "serviceAbnormal"             子模块状态异常。
"""


from agora_rest_client.core import exceptions

class RequestObject(object):
    """
    Request object
    """

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            if not hasattr(self, k):
                raise exceptions.AttributeException("{} has no attribute '{}'".format(type(self).__name__, k))

            setattr(self, k, v)

    def __str__(self):
        return ",".join(["{}={}".format(k, v) for k, v in self.__dict__.items()])

    def to_dict(self):
        """
        Returns the object properties as a dict
        """
        result = {}

        attrs = [attr for attr in dir(self) if not callable(getattr(self, attr)) and not attr.startswith("__")]
        for attr in attrs:
            value = getattr(self, attr)

            if value is None:
                continue

            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_json(self):
        """
        Returns the object properties as a json
        """
        return self.to_dict()




from agora_rest_client.core import request
from agora_rest_client.core import response

"""
官方文档: https://doc.shengwang.cn/api-ref/cloud-recording/restful/cloud-recording/operations/get-v1-apps-appid-cloud_recording-resourceid-resourceid-sid-sid-mode-mode-query
开始录制后, 你可以调用 query 方法查询录制状态.
对于每个声网账号, 每秒钟的请求数限制为 10 次. 如需提高此限制, 请联系技术支持.
query 请求仅在会话内有效. 如果录制启动错误, 或录制已结束, 调用 query 将返回 404.
"""

class FileList(response.ResponseObject):
    """
    type: string

    录制产生的 M3U8 文件和 MP4 文件的文件名.
    """
    filename = None

    """
    type: number

    该文件的录制开始时间, Unix 时间戳, 单位为毫秒.
    """
    sliceStartTime = None

class Payload(response.ResponseObject):
    """
    type: array[object]

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_query.FileList`
    """
    fileList = None

    """
    type: boolean

    页面录制是否处于暂停状态:
    """
    onhold = None

    """
    type: string

    将订阅内容上传至扩展服务的状态:
    "init": 服务正在初始化.
    "inProgress": 服务启动完成, 正在进行中.
    "exit": 服务退出.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.ExtensionServiceState`
    """
    state = None

class ExtensionServiceState(response.ResponseObject):
    """
    type: object

    页面录制时会返回如下字段

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_query.Payload`
    """
    payload = None

    """
    type: string

    扩展服务的名称:
    web_recorder_service: 代表扩展服务为页面录制.
    rtmp_publish_service: 代表扩展服务为转推页面录制到 CDN.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.ExtensionServiceName`
    """
    serviceName = None

class ServerResponse(response.ResponseObject):
    """
    type: number

    当前云服务的状态:
    0: 没有开始云服务.
    1: 云服务初始化完成.
    2: 云服务组件开始启动.
    3: 云服务部分组件启动完成.
    4: 云服务所有组件启动完成.
    5: 云服务正在进行中.
    6: 云服务收到停止请求.
    7: 云服务所有组件均停止.
    8: 云服务已退出.
    20: 云服务异常退出.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.ServiceStatus`
    """
    status = None

    """
    type: array[object]

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_query.ExtensionServiceState`
    """
    extensionServiceState = None

class RequestPathParamsApiQuery(request.RequestObject):
    """
    type: required string

    录制模式:
    individual: 单流录制模式.
    mix: 合流录制模式.
    web: 页面录制模式.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.Mode`
    """
    mode = None

    """
    type: required string

    通过 acquire 请求获取到的 Resource ID.
    """
    resource_id = None

    """
    type: required string

    通过 start 获取的录制 ID.
    """
    sid = None

class ResponseApiQuery(response.ResponseObject):
    """
    type: string

    云端录制使用的 Resource ID.
    """
    resourceId = None

    """
    type: string

    录制 ID.
    """
    sid = None

    """
    type: object

    页面录制模式下会返回的字段.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_query.ServerResponse`
    """
    serverResponse = None

    """
    type: string

    录制的频道名.
    """
    cname = None

    """
    type: string

    字符串内容为云端录制服务在 RTC 频道内使用的 UID, 用于标识频道内的录制服务.
    """
    uid = None

def api_query(client, request_path_params_obj, response_obj=ResponseApiQuery, trace_id=None):
    """
    Query the recording status
    查询云端录制状态

    :type client: object
    :param client: CloudRecordingClient object

    :type request_path_params_obj: object
    :param request_path_params_obj: request path params object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_query.RequestPathParamsApiQuery`

    :type response_obj: object
    :param response_obj: response object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_query.ResponseApiQuery`

    :type trace_id: string
    :param trace_id: trace id

    :return: response object ResponseApiQuery
    """
    url = '/v1/apps/{}/cloud_recording/resourceid/{}/sid/{}/mode/{}/query'.format(client.app_id, request_path_params_obj.resource_id, request_path_params_obj.sid, request_path_params_obj.mode)
    client.logger.debug("url:%s", url)

    return client.call_api('GET', url, response_obj=response_obj, trace_id=trace_id)



from agora_rest_client.services.cloud_recording.v1.cloud_recording_client import CloudRecordingClient
from agora_rest_client.services.cloud_recording.v1.web_recording.api_acquire import web_recording_acquire
from agora_rest_client.services.cloud_recording.v1.web_recording.api_query import web_recording_query
from agora_rest_client.services.cloud_recording.v1.web_recording.api_start import web_recording_start
from agora_rest_client.services.cloud_recording.v1.web_recording.api_stop import web_recording_stop
from agora_rest_client.services.cloud_recording.v1.web_recording.api_update import web_recording_update

class WebRecordingClient(CloudRecordingClient):
    """
    Web recording client
    """

    def __init__(self):
        super(WebRecordingClient, self).__init__()

    def new_builder():
        return WebRecordingClient()

    def acquire(self, cname, uid, resource_expired_hour=None, exclude_resource_ids=None, region_affinity=None, trace_id=None):
        """
        Web recoding acquire
        获取云端录制资源

        :type client: object
        :param client: WebRecordingClient object

        :type cname: str
        :param cname: cname
        :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.RequestBodyApiAcquire.cname`

        :type uid: str
        :param uid: uid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.RequestBodyApiAcquire.uid`

        :type resource_expired_hour: int
        :param resource_expired_hour: resource expired hour
        :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.ClientRequest.resourceExpiredHour`

        :type exclude_resource_ids: list
        :param exclude_resource_ids: exclude resource ids
        :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.ClientRequest.excludeResourceIds`

        :type region_affinity: int
        :param region_affinity: region affinity
        :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.ClientRequest.regionAffinity`
        :value: enum of `agora_rest_client.services.cloud_recording.v1.api.RegionAffinity`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiAcquire
        """
        return web_recording_acquire(self, cname, uid, resource_expired_hour=resource_expired_hour, exclude_resource_ids=exclude_resource_ids,
                                     region_affinity=region_affinity, trace_id=trace_id)

    def query(self, resource_id, sid, trace_id=None):
        """
        Web recording query
        查询云端录制状态

        :type client: object
        :param client: WebRecordingClient object

        :type resource_id: str
        :param resource_id: resource id
        :refer: `agora_rest_client.services.cloud_recording.v1.api_query.RequestPathParamsApiQuery.resource_id`

        :type sid: str
        :param sid: sid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_query.RequestPathParamsApiQuery.sid`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiQuery
        """
        return web_recording_query(self, resource_id, sid, trace_id=trace_id)

    def start(self, resource_id, cname, uid, storage_config, extension_service_config, trace_id=None):
        """
        Web recording start
        开始云端录制

        :type client: object
        :param client: WebRecordingClient object

        :type resource_id: str
        :param resource_id: resource id
        :refer: `agora_rest_client.services.cloud_recording.v1.api_start.RequestPathParamsApiStart.resource_id`

        :type cname: str
        :param cname: cname
        :refer: `agora_rest_client.services.cloud_recording.v1.api_start.RequestBodyApiStart.cname`

        :type uid: str
        :param uid: uid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_start.RequestBodyApiStart.uid`

        :type storage_config: object
        :param storage_config: storage config
        :refer: `agora_rest_client.services.cloud_recording.v1.api_start.StorageConfig`
        :value: instance of `agora_rest_client.services.cloud_recording.v1.web_recording.api_start.StorageConfig`

        :type extension_service_config: object
        :param extension_service_config: extension service config
        :refer: `agora_rest_client.services.cloud_recording.v1.api_start.ExtensionServiceConfig`
        :value: instance of `agora_rest_client.services.cloud_recording.v1.web_recording.api_start.ExtensionServiceConfig`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiStart
        """
        return web_recording_start(self, resource_id, cname, uid, storage_config, extension_service_config, trace_id=trace_id)

    def stop(self, resource_id, sid, cname, uid, async_stop=False, trace_id=None):
        """
        Web recording stop
        停止云端录制

        :type client: object
        :param client: WebRecordingClient object

        :type resource_id: str
        :param resource_id: resource id
        :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.RequestPathParamsApiStop.resource_id`

        :type sid: str
        :param sid: sid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.RequestPathParamsApiStop.sid`

        :type cname: str
        :param cname: cname
        :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.RequestBodyApiStop.cname`

        :type uid: str
        :param uid: uid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.RequestBodyApiStop.uid`

        :type async_stop: bool
        :param async_stop: async stop
        :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.ClientRequest.async_stop`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiStop
        """
        return web_recording_stop(self, resource_id, sid, cname, uid, async_stop=async_stop, trace_id=trace_id)

    def update(self, resource_id, sid, cname, uid, web_recording_config=None, rtmp_publish_config=None, trace_id=None):
        """
        Web recording update
        更新云端录制设置

        :type client: object
        :param client: WebRecordingClient object

        :type resource_id: str
        :param resource_id: resource id
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update.RequestPathParamsApiUpdate.resource_id`

        :type sid: str
        :param sid: sid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update.RequestPathParamsApiUpdate.sid`

        :type cname: str
        :param cname: cname
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update.RequestBodyApiUpdate.cname`

        :type uid: str
        :param uid: uid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update.RequestBodyApiUpdate.uid`

        :type web_recording_config: object
        :param web_recording_config: web recording config
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update.WebRecordingConfig`
        :value: instance of `agora_rest_client.services.cloud_recording.v1.web_recording.api_update.WebRecordingConfig`

        :type rtmp_publish_config: object
        :param rtmp_publish_config: rtmp publish config
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update.RtmpPublishConfig`
        :value: instance of `agora_rest_client.services.cloud_recording.v1.web_recording.api_update.RtmpPublishConfig`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiUpdate
        """
        return web_recording_update(self, resource_id, sid, cname, uid, web_recording_config=web_recording_config,
                                    rtmp_publish_config=rtmp_publish_config, trace_id=trace_id)



from enum import Enum

# Amazon S3 加密模式, Server-Side Encryption
class AmazonS3Sse(Enum):
    # KMS 加密
    KMS = 'kms'
    # AES256 加密
    AES_256 = 'aes256'

# 音频声道数
class AudioChannels(Enum):
    # 单声道
    MONO = '1'
    # 双声道
    STEREO = '2'

# 音频采样率
class AudioSampleRate(Enum):
    # 48 kHz
    SAMPLE_RATE_48 = '48000'
    # 32 kHz
    SAMPLE_RATE_32 = '32000'
    # 16 kHz
    SAMPLE_RATE_16 = '16000'

# 输出音频的采样率/码率/编码模式和声道数
class AudioProfile(Enum):
    # 48 kHz 采样率, 音乐编码, 单声道, 编码码率约 48 Kbps
    SAMPLE_RATE_48_MONO_ENCODING_48 = 0
    # 48 kHz 采样率, 音乐编码, 单声道, 编码码率约 128 Kbps
    SAMPLE_RATE_48_MONO_ENCODING_128 = 1
    # 48 kHz 采样率, 音乐编码, 双声道, 编码码率约 192 Kbps
    SAMPLE_RATE_48_STEREO_ENCODING_192 = 2

# 视频文件类型
class AvFileType(Enum):
    HLS = 'hls'
    MP4 = 'mp4'

# 频道场景
class ChannelType(Enum):
    # 通信场景
    COMMUNICATION = 0
    # 直播场景
    LIVE = 1

# 各云端录制应用的组合方式
class CombinationPolicy(Enum):
    # 如需延时转码或延时混音, 则选用此种方式
    POSTPONE_TRANSCODING = 'postpone_transcoding'
    # 除延时转码和延时混音外, 均选用此种方式
    DEFAULT = 'default'

# 文件的容器格式
class ContainerFormat(Enum):
    # 延时转码时的默认格式
    MP4 = 'mp4'
    # 延时混音时的默认格式
    MP3 = 'mp3'
    # M4A 格式
    M4A = 'm4a'
    # AAC 格式
    AAC= 'aac'

# 解密模式
class DecryptionMode(Enum):
    # 不加密
    NO_ENCRYPTION = 0
    # AES_128_XTS 加密模式
    AES_128_XTS = 1
    # AES_128_ECB 加密模式
    AES_128_ECB = 2
    # AES_256_XTS 加密模式
    AES_256_XTS = 3
    # SM4_128_ECB 加密模式
    SM4_128_ECB = 4
    # AES_128_GCM 加密模式
    AES_128_GCM = 5
    # AES_256_GCM 加密模式
    AES_256_GCM = 6
    # AES_128_GCM2 加密模式
    AES_128_GCM2 = 7
    # AES_256_GCM2 加密模式
    AES_256_GCM2 = 8

# 扩展服务内的错误处理策略
class ExtensionErrorHandlePolicy(Enum):
    # 当前扩展服务出错时, 停止其他扩展服务
    ERROR_ABORT = 'error_abort'
    # 当前扩展服务出错时, 其他扩展服务不受影响
    ERROR_IGNORE = 'error_ignore'

# 扩展服务的名称
class ExtensionServiceName(Enum):
    # 代表扩展服务为页面录制
    WEB_RECORDER_SERVICE = 'web_recorder_service'
    # 代表扩展服务为转推页面录制到 CDN
    RTMP_PUBLISH_SERVICE = 'rtmp_publish_service'

# 将订阅内容上传至扩展服务的状态
class ExtensionServiceState(Enum):
    # 服务正在初始化
    INIT = 'init'
    # 服务启动完成, 正在进行中
    IN_PROGRESS = 'inProgress'
    # 服务退出
    EXIT = 'exit'

# 服务类型
class ExtensionServiceStateName(Enum):
    # 上传服务
    UPLOAD_SERVICE = 'upload_service'
    # 页面录制服务
    WEB_RECORDER_SERVICE = 'web_recorder_service'

# 视频合流布局
class MixedVideoLayout(Enum):
    # 悬浮布局
    FLOATING = 0
    # 自适应布局
    ADAPTIVE = 1
    # 垂直布局
    VERTICAL = 2
    # 自定义布局
    CUSTOM = 3

# 录制模式
class Mode(Enum):
    # 单流录制模式
    INDIVIDUAL = 'individual'
    # 合流录制模式
    MIX = 'mix'
    # 页面录制模式
    WEB = 'web'

# 延时转码模式
class PostponeTranscodeMode(Enum):
    # 延时转码
    POSTPONE_TRANSCODING = 'postponeTranscoding'
    # 延时混音
    AUDIO_MIX = 'audioMix'

# 指定使用某个区域的资源进行录制
class RegionAffinity(Enum):
    # 根据发起请求的区域就近调用资源
    NEAR = 0
    # 中国
    CHINA = 1
    # 东南亚
    SOUTHEAST_ASIA = 2
    # 欧洲
    EUROPE = 3
    # 北美
    NORTH_AMERICA = 4

# 画面显示模式
class RenderMode(Enum):
    # 裁剪模式
    CROP = 0
    # 缩放模式
    SCALE = 1

# 云端录制资源使用场景
class Scene(Enum):
    # 实时音视频录制
    RTC = 0
    # 页面录制
    WEB = 1
    # 单流录制模式, 且开启延时转码或延时混音
    INDIVIDUAL_POSTPONE = 2

# 当前云服务的状态
class ServiceStatus(Enum):
    # 没有开始云服务
    NOT_STARTED = 0
    # 云服务初始化完成
    INITIALIZED = 1
    # 云服务组件开始启动
    STARTING = 2
    # 云服务部分组件启动完成
    PARTIAL_STARTED = 3
    # 云服务所有组件启动完成
    ALL_STARTED = 4
    # 云服务正在进行中
    IN_PROGRESS = 5
    # 云服务收到停止请求
    STOP_REQUESTED = 6
    # 云服务所有组件均停止
    ALL_STOPPED = 7
    # 云服务已退出
    EXITED = 8
    # 云服务异常退出
    EXCEPTION_EXITED = 20

# 截图文件类型
class SnapshotFileType(Enum):
    JPG = 'jpg'

# 视频截图类型
class SnapshotType(Enum):
    # 仅截图, 不录制
    SNAPSHOT_NO_RECORDING = 0
    # 录制并截图
    SNAPSHOT_AND_RECORDING = 1

# 第三方云存储平台
class StorageVendor(Enum):
    # Amazon S3
    AMAZON_S3 = 1
    # 阿里云
    ALIYUN_OSS = 2
    # 腾讯云
    TENCENT_CLOUD = 3
    # Microsoft Azure
    MICROSOFT_AZURE = 5
    # 谷歌云
    GOOGLE_CLOUD = 6
    # 华为云
    HUAWEI_CLOUD = 7
    # 百度智能云
    BAIDU_CLOUD = 8

# 媒体流的输出模式
class StreamMode(Enum):
    # 默认模式
    DEFAULT = 'default'
    # 标准模式
    STANDARD = 'standard'
    # 原始编码模式
    ORIGINAL = 'original'

# 订阅的媒体流类型
class StreamTypes(Enum):
    # 仅订阅音频
    AUDIO_ONLY = 0
    # 仅订阅视频
    VIDEO_ONLY = 1
    # 订阅音频和视频
    AUDIO_AND_VIDEO = 2

# 预估的订阅人数峰值
class SubscribeUidGroup(Enum):
    # 1 到 2 个 UID
    UID_1_2 = 0
    # 3 到 7 个 UID
    UID_3_7 = 1
    # 8 到 12 个 UID
    UID_8_12 = 2
    # 13 到 17 个 UID
    UID_13_17 = 3
    # 17 到 32 个 UID
    UID_17_32 = 4
    # 32 到 49 个 UID
    UID_32_49 = 5

# 当前录制上传的状态
class UploadingStatus(Enum):
    # 本次录制的文件已经全部上传至指定的第三方云存储
    UPLOADED = 'uploaded'
    # 本次录制的文件已经全部上传完成, 但是至少有一个 TS 文件上传到了声网备份云. 声网服务器会自动将这部分文件继续上传至指定的第三方云存储
    BACKED_UP = 'backuped'
    # 未知状态
    UNKNOWN = 'unknown'

# 订阅的视频流类型
class VideoStreamType(Enum):
    # 视频大流
    BIG_STREAM = 0
    # 视频小流
    SMALL_STREAM = 1




import json
import time
import uuid
from agora_rest_client.core import exceptions
from agora_rest_client.core.client import Client
from agora_rest_client.services.cloud_recording.v1.api_acquire import api_acquire
from agora_rest_client.services.cloud_recording.v1.api_query import api_query
from agora_rest_client.services.cloud_recording.v1.api_start import api_start
from agora_rest_client.services.cloud_recording.v1.api_stop import api_stop
from agora_rest_client.services.cloud_recording.v1.api_update import api_update
from agora_rest_client.services.cloud_recording.v1.api_update_layout import api_update_layout

class CloudRecordingClient(Client):
    """
    Cloud recording client
    """

    def __init__(self):
        super(CloudRecordingClient, self).__init__()

        self._error_code_key = 'code'
        self._error_msg_key = 'reason'

    def new_builder():
        return CloudRecordingClient()

    def call_api(self, method, url, params=None, post_data=None, post_json=None, headers=None, timeout_seconds=5, response_obj=None, trace_id=None, is_retry=False):
        """
        Call api

        :type method: str
        :param method: http method

        :type url: str
        :param url: http url

        :type params: object
        :param params: http params

        :type post_data: object
        :param post_data: http post data

        :type post_json: object
        :param post_json: http post json

        :type headers: object
        :param headers: http headers

        :type timeout_seconds: int
        :param timeout_seconds: http timeout

        :type response_obj: object
        :param response_obj: response object

        :type trace_id: string
        :param trace_id: trace id

        :type is_retry: boolean
        :param is_retry: is retry

        :type: object
        :return: instance of `response_obj`
        """
        if trace_id is None:
            trace_id = uuid.uuid1()

        status_code = None
        error_code = None
        error_msg = None
        http_retry_count = self._http_retry_count if is_retry else 1

        # Retry
        for retry in range(http_retry_count):
            retry_num = retry + 1

            try:
                resp = super().call_api(method, url, params=params, post_data=post_data, post_json=post_json, headers=headers, timeout_seconds=timeout_seconds, trace_id=trace_id)
                status_code = resp.status_code

                self._logger.debug('call api, trace_id:%s, url:%s, is_retry:%s, http_retry_count:%d, retry_num:%d, status_code:%s', trace_id, url, is_retry, http_retry_count, retry_num, status_code)

                # Request success
                if status_code == 200 or status_code == 201:
                    return response_obj(**json.loads(resp.text))

                try:
                    resp_json = resp.json()
                    error_code = resp_json.get(self._error_code_key)
                    error_msg = resp_json.get(self._error_msg_key) if resp_json.get(self._error_msg_key) is not None else resp.text
                except Exception as e:
                    error_msg = resp.text

                # Request failed
                # No need to retry
                if status_code >= 400 and status_code < 410:
                    self._logger.error('call api, no retry, trace_id:%s, url:%s, retry_num:%d, status_code:%s, error_code:%s, error_msg:%s', trace_id, url, retry_num, status_code, error_code, error_msg)
                    raise exceptions.ServiceResponseException(status_code, error_code, error_msg)
            except exceptions.ClientTimeoutException as e:
                error_msg = '%s' % e
                self._logger.error('call api, timeout, trace_id:%s, url:%s, retry_num:%d', trace_id, url, retry_num)
            except exceptions.ClientRequestException as e:
                error_msg = '%s' % e
                self._logger.error('call api, request failed, err:%s, trace_id:%s, url:%s, retry_num:%d', e, trace_id, url, retry_num)

            if is_retry:
                # Retry, sleep
                sleep_second = retry_num
                time.sleep(sleep_second)
                self._logger.debug('call api, retry, trace_id:%s, url:%s, retry_num:%d, status_code:%s, error_code:%s, error_msg:%s, sleep_second:%d', trace_id, url, retry_num, status_code, error_code, error_msg, sleep_second)

        raise exceptions.ClientRequestException(status_code, error_code, error_msg)

    def acquire(self, request_body_obj, trace_id=None):
        """
        Acquire a resource id
        获取云端录制资源

        :type client: object
        :param client: CloudRecordingClient object

        :type request_body_obj: object
        :param request_body_obj: request body object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_acquire.RequestBodyApiAcquire`

        :type response_obj: object
        :param response_obj: response object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_acquire.ResponseApiAcquire`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiAcquire
        """
        return api_acquire(self, request_body_obj, trace_id=trace_id)

    def query(self, request_path_params_obj, trace_id=None):
        """
        Query the recording status
        查询云端录制状态

        :type client: object
        :param client: CloudRecordingClient object

        :type request_path_params_obj: object
        :param request_path_params_obj: request path params object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_query.RequestPathParamsApiQuery`

        :type response_obj: object
        :param response_obj: response object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_query.ResponseApiQuery`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiQuery
        """
        return api_query(self, request_path_params_obj, trace_id=trace_id)

    def start(self, request_path_params_obj, request_body_obj, trace_id=None):
        """
        Start recording
        开始云端录制

        :type client: object
        :param client: CloudRecordingClient object

        :type request_path_params_obj: object
        :param request_path_params_obj: request path params object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.RequestPathParamsApiStart`

        :type request_body_obj: object
        :param request_body_obj: request body object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.RequestBodyApiStart`

        :type response_obj: object
        :param response_obj: response object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.ResponseApiStart`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiStart
        """
        return api_start(self, request_path_params_obj, request_body_obj, trace_id=trace_id)

    def stop(self, request_path_params_obj, request_body_obj, trace_id=None):
        """
        Stop recording
        停止云端录制

        :type client: object
        :param client: CloudRecordingClient object

        :type request_path_params_obj: object
        :param request_path_params_obj: request path params object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_stop.RequestPathParamsApiStop`

        :type request_body_obj: object
        :param request_body_obj: request body object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_stop.RequestBodyApiStop`

        :type response_obj: object
        :param response_obj: response object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_stop.ResponseApiStop`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiStop
        """
        return api_stop(self, request_path_params_obj, request_body_obj, trace_id=trace_id)

    def update(self, request_path_params_obj, request_body_obj, trace_id=None):
        """
        Update recording
        更新云端录制设置

        :type client: object
        :param client: CloudRecordingClient object

        :type request_path_params_obj: object
        :param request_path_params_obj: request path params object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update.RequestPathParamsApiUpdate`

        :type request_body_obj: object
        :param request_body_obj: request body object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update.RequestBodyApiUpdate`

        :type response_obj: object
        :param response_obj: response object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update.ResponseApiUpdate`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiUpdate
        """
        return api_update(self, request_path_params_obj, request_body_obj, trace_id=trace_id)

    def update_layout(self, request_path_params_obj, request_body_obj, trace_id=None):
        """
        Update layout of the recording
        更新云端录制合流布局

        :type client: object
        :param client: CloudRecordingClient object

        :type request_path_params_obj: object
        :param request_path_params_obj: request path params object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update_layout.RequestPathParamsApiUpdateLayout`

        :type request_body_obj: object
        :param request_body_obj: request body object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update_layout.RequestBodyApiUpdateLayout`

        :type response_obj: object
        :param response_obj: response object
        :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update_layout.ResponseApiUpdateLayout`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiUpdateLayout
        """
        return api_update_layout(self, request_path_params_obj, request_body_obj, trace_id=trace_id)



import operator
import schedule
import socket
import threading
import time
from enum import Enum
from agora_rest_client.core import exceptions

# China domain
CHINA_MAINLAND_MAJOR_DOMAIN = 'sd-rtn.com'
# Oversea domain
OVERSEA_MAJOR_DOMAIN = 'agora.io'

# Asia & Pacific
AP_NORTHEAST_REGION_DOMAIN_PREFIX = 'api-ap-northeast-1'
AP_SOUTHEAST_REGION_DOMAIN_PREFIX = 'api-ap-southeast-1'

# China
CN_EAST_REGION_DOMAIN_PREFIX = 'api-cn-east-1'
CN_NORTH_REGION_DOMAIN_PREFIX = 'api-cn-north-1'

# European Union
EU_CENTRAL_REGION_DOMAIN_PREFIX = 'api-eu-central-1'
EU_WEST_REGION_DOMAIN_PREFIX = 'api-eu-west-1'

# United States
US_EAST_REGION_DOMAIN_PREFIX = 'api-us-east-1'
US_WEST_REGION_DOMAIN_PREFIX = 'api-us-west-1'

# Region area
class RegionArea(Enum):
    US = 0
    EU = 1
    AP = 2
    CN = 3

# Region domain
REGION_DOMAIN = {
    RegionArea.US.value: {
        'prefixes': [US_WEST_REGION_DOMAIN_PREFIX, US_EAST_REGION_DOMAIN_PREFIX],
        'suffixes': [OVERSEA_MAJOR_DOMAIN, CHINA_MAINLAND_MAJOR_DOMAIN],
    },
    RegionArea.EU.value: {
        'prefixes': [EU_WEST_REGION_DOMAIN_PREFIX, EU_CENTRAL_REGION_DOMAIN_PREFIX],
        'suffixes': [OVERSEA_MAJOR_DOMAIN, CHINA_MAINLAND_MAJOR_DOMAIN],
    },
    RegionArea.AP.value: {
        'prefixes': [AP_SOUTHEAST_REGION_DOMAIN_PREFIX, AP_NORTHEAST_REGION_DOMAIN_PREFIX],
        'suffixes': [OVERSEA_MAJOR_DOMAIN, CHINA_MAINLAND_MAJOR_DOMAIN],
    },
    RegionArea.CN.value: {
        'prefixes': [CN_EAST_REGION_DOMAIN_PREFIX, CN_NORTH_REGION_DOMAIN_PREFIX],
        'suffixes': [CHINA_MAINLAND_MAJOR_DOMAIN, OVERSEA_MAJOR_DOMAIN],
    },
}

# Select best domain, interval seconds
SELECT_BEST_DOMAIN_INTERVAL_SECONDS = 30

class Domain(object):
    """
    Domain class
    """

    def __init__(self):
        self._domain_list = []
        self._domain_list_running = []
        self._logger = None
        self._region = None

    def get_domain_list(self):
        return self._domain_list

    def get_region(self):
        return self._region

    def job_select_best_domain(self):
        """
        Scheduled job, select best domain
        """
        schedule.every(SELECT_BEST_DOMAIN_INTERVAL_SECONDS).seconds.do(self.select_best_domain)
        while True:
            schedule.run_pending()
            time.sleep(1)

    def resolve_domain(self, host, host_suffix):
        """
        Resolve domain

        :type host: str
        :param host: domain

        :type host_suffix: str
        :param host_suffix: domain suffix

        :return: ip address or None
        """
        try:
            time_start = time.time()
            ip_address = socket.gethostbyname(host)
            duration_ms = (time.time()-time_start)*1000

            self._domain_list_running.append({'host': host, 'host_suffix': host_suffix, 'duration_ms': duration_ms})
            self._logger.debug('resolve domain, host:%s, duration:%0.2fms, ip_address:%s', host, duration_ms, ip_address)

            return ip_address
        except Exception as e:
            self._logger.error('resolve domain failed:%s', e)
            return None

    def run(self, logger):
        """
        Run

        :type logger: object
        :param logger: logger
        """
        # Avoid duplicate run
        if self._logger is not None:
            return

        self._logger = logger
        self.select_best_domain()

        # Start scheduled job
        t = threading.Thread(target=self.job_select_best_domain, args=())
        t.start()

    def select_best_domain(self):
        """
        Select best domain
        """
        threads = []
        self._domain_list_running = []
        self._logger.debug('select best domain start, region:%s, domain_list:%s, domain_list_running:%s', self._region, self._domain_list, self._domain_list_running)

        for suffix in REGION_DOMAIN[self._region]['suffixes']:
            host = '%s.%s' % (REGION_DOMAIN[self._region]['prefixes'][0], suffix)
            t = threading.Thread(target=self.resolve_domain, args=(host, suffix))
            threads.append(t)
            t.start()

        for t in threads:
            t.join()

        # Check domain
        if len(self._domain_list_running) == 0:
            self._logger.error('select best domain failed, no available domain, region:%s, domain_list:%s, domain_list_running:%s', self._region, self._domain_list, self._domain_list_running)
            return

        # Sort by duration_ms
        domain_list_running = sorted(self._domain_list_running, key=operator.itemgetter('duration_ms'))
        domain_list = []

        for prefix in REGION_DOMAIN[self._region]['prefixes']:
            domain_list.append('%s.%s' % (prefix, domain_list_running[0]['host_suffix']))

        self._domain_list = domain_list
        self._logger.debug('select best domain end, region:%s, domain_list:%s, domain_list_running:%s', self._region, self._domain_list, self._domain_list_running)

    def set_logger(self, logger):
        self._logger = logger

    def set_region(self, region):
        self._region = region

# Default domain
default_domain = Domain()


from agora_rest_client.services.cloud_recording.v1.cloud_recording_client import CloudRecordingClient
from agora_rest_client.services.cloud_recording.v1.mix_recording.api_acquire import mix_recording_acquire
from agora_rest_client.services.cloud_recording.v1.mix_recording.api_query import mix_recording_query
from agora_rest_client.services.cloud_recording.v1.mix_recording.api_start import mix_recording_start
from agora_rest_client.services.cloud_recording.v1.mix_recording.api_stop import mix_recording_stop
from agora_rest_client.services.cloud_recording.v1.mix_recording.api_update_layout import mix_recording_update_layout
from agora_rest_client.services.cloud_recording.v1.mix_recording.api_update import mix_recording_update

class MixRecordingClient(CloudRecordingClient):
    """
    Mix recording client
    """

    def __init__(self):
        super(MixRecordingClient, self).__init__()

    def new_builder():
        return MixRecordingClient()

    def acquire(self, cname, uid, resource_expired_hour=None, exclude_resource_ids=None, region_affinity=None, trace_id=None):
        """
        Mix recording acquire
        获取云端录制资源

        :type client: object
        :param client: MixRecordingClient object

        :type cname: str
        :param cname: cname
        :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.RequestBodyApiAcquire.cname`

        :type uid: str
        :param uid: uid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.RequestBodyApiAcquire.uid`

        :type resource_expired_hour: int
        :param resource_expired_hour: resource expired hour
        :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.ClientRequest.resourceExpiredHour`

        :type exclude_resource_ids: list
        :param exclude_resource_ids: exclude resource ids
        :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.ClientRequest.excludeResourceIds`

        :type region_affinity: int
        :param region_affinity: region affinity
        :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.ClientRequest.regionAffinity`
        :value: enum of `agora_rest_client.services.cloud_recording.v1.api.RegionAffinity`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiAcquire
        """
        return mix_recording_acquire(self, cname, uid, resource_expired_hour=resource_expired_hour, exclude_resource_ids=exclude_resource_ids,
                                     region_affinity=region_affinity, trace_id=trace_id)

    def query(self, resource_id, sid, trace_id=None):
        """
        Mix recording query
        查询云端录制状态

        :type client: object
        :param client: MixRecordingClient object

        :type resource_id: str
        :param resource_id: resource id
        :refer: `agora_rest_client.services.cloud_recording.v1.api_query.RequestPathParamsApiQuery.resource_id`

        :type sid: str
        :param sid: sid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_query.RequestPathParamsApiQuery.sid`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiQuery
        """
        return mix_recording_query(self, resource_id, sid, trace_id=trace_id)

    def start(self, resource_id, cname, uid, token, storage_config, recording_config=None, trace_id=None):
        """
        Mix recording start
        开始云端录制

        :type client: object
        :param client: MixRecordingClient object

        :type resource_id: str
        :param resource_id: resource id
        :refer: `agora_rest_client.services.cloud_recording.v1.api_start.RequestPathParamsApiStart.resource_id`

        :type cname: str
        :param cname: cname
        :refer: `agora_rest_client.services.cloud_recording.v1.api_start.RequestBodyApiStart.cname`

        :type uid: str
        :param uid: uid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_start.RequestBodyApiStart.uid`

        :type token: str
        :param token: token
        :refer: `agora_rest_client.services.cloud_recording.v1.api_start.ClientRequest.token`

        :type storage_config: object
        :param storage_config: storage config
        :refer: `agora_rest_client.services.cloud_recording.v1.api_start.StorageConfig`
        :value: instance of `agora_rest_client.services.cloud_recording.v1.mix_recording.api_start.StorageConfig`

        :type recording_config: object
        :param recording_config: recording config
        :refer: `agora_rest_client.services.cloud_recording.v1.api_start.RecordingConfig`
        :value: instance of `agora_rest_client.services.cloud_recording.v1.mix_recording.api_start.RecordingConfig`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiStart
        """
        return mix_recording_start(self, resource_id, cname, uid, token, storage_config, recording_config=recording_config, trace_id=trace_id)

    def stop(self, resource_id, sid, cname, uid, async_stop=False, trace_id=None):
        """
        Mix recording stop
        停止云端录制

        :type client: object
        :param client: MixRecordingClient object

        :type resource_id: str
        :param resource_id: resource id
        :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.RequestPathParamsApiStop.resource_id`

        :type sid: str
        :param sid: sid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.RequestPathParamsApiStop.sid`

        :type cname: str
        :param cname: cname
        :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.RequestBodyApiStop.cname`

        :type uid: str
        :param uid: uid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.RequestBodyApiStop.uid`

        :type async_stop: bool
        :param async_stop: async stop
        :refer: `agora_rest_client.services.cloud_recording.v1.api_stop.ClientRequest.async_stop`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiStop
        """
        return mix_recording_stop(self, resource_id, sid, cname, uid, async_stop=async_stop, trace_id=trace_id)

    def update(self, resource_id, sid, cname, uid, stream_subscribe=None, trace_id=None):
        """
        Mix recording update
        更新云端录制设置

        :type client: object
        :param client: MixRecordingClient object

        :type resource_id: str
        :param resource_id: resource id
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update.RequestPathParamsApiUpdate.resource_id`

        :type sid: str
        :param sid: sid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update.RequestPathParamsApiUpdate.sid`

        :type cname: str
        :param cname: cname
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update.RequestBodyApiUpdate.cname`

        :type uid: str
        :param uid: uid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update.RequestBodyApiUpdate.uid`

        :type stream_subscribe: object
        :param stream_subscribe: stream subscribe
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update.StreamSubscribe`
        :value: instance of `agora_rest_client.services.cloud_recording.v1.mix_recording.api_update.StreamSubscribe`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiUpdate
        """
        return mix_recording_update(self, resource_id, sid, cname, uid, stream_subscribe=stream_subscribe, trace_id=trace_id)

    def update_layout(self, resource_id, sid, cname, uid, max_resolution_uid=None, mixed_video_layout=None, background_color=None,
                      background_image=None, default_user_background_image=None, layout_config=None, background_config=None, trace_id=None):
        """
        Mix recording update layout
        更新云端录制合流布局

        :type client: object
        :param client: MixRecordingClient object

        :type resource_id: str
        :param resource_id: resource id
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.RequestPathParamsApiUpdate.resource_id`

        :type sid: str
        :param sid: sid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.RequestPathParamsApiUpdate.sid`

        :type cname: str
        :param cname: cname
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.RequestBodyApiUpdate.cname`

        :type uid: str
        :param uid: uid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.RequestBodyApiUpdate.uid`

        :type max_resolution_uid: str
        :param max_resolution_uid: max resolution uid
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.maxResolutionUid`

        :type mixed_video_layout: int
        :param mixed_video_layout: mixed video layout
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.mixedVideoLayout`

        :type background_color: str
        :param background_color: background color
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.backgroundColor`

        :type background_image: str
        :param background_image: background image
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.backgroundImage`

        :type default_user_background_image: str
        :param default_user_background_image: default user background image
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.defaultUserBackgroundImage`

        :type layout_config: object
        :param layout_config: layout config
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.LayoutConfig`
        :value: instance of `agora_rest_client.services.cloud_recording.v1.mix_recording.api_update_layout.LayoutConfig`

        :type background_config: object
        :param background_config: background config
        :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.BackgroundConfig`
        :value: instance of `agora_rest_client.services.cloud_recording.v1.mix_recording.api_update_layout.BackgroundConfig`

        :type trace_id: string
        :param trace_id: trace id

        :return: response object ResponseApiUpdate
        """
        return mix_recording_update_layout(self, resource_id, sid, cname, uid, max_resolution_uid=max_resolution_uid, mixed_video_layout=mixed_video_layout,
                                           background_color=background_color, background_image=background_image, default_user_background_image=default_user_background_image,
                                           layout_config=layout_config, background_config=background_config, trace_id=trace_id)



from agora_rest_client.services.cloud_recording.v1 import api_acquire
from agora_rest_client.services.cloud_recording.v1.api import Scene

class ClientRequest(api_acquire.ClientRequest):
    pass

class RequestBodyApiAcquire(api_acquire.RequestBodyApiAcquire):
    pass

class ResponseApiAcquire(api_acquire.ResponseApiAcquire):
    pass

def individual_recording_acquire(client, cname, uid, enable_postpone_transcoding_mix=False, resource_expired_hour=None, exclude_resource_ids=None, region_affinity=None, trace_id=None):
    """
    Individual recording acquire
    获取云端录制资源

    :type client: object
    :param client: IndividualRecordingClient object

    :type cname: str
    :param cname: cname
    :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.RequestBodyApiAcquire.cname`

    :type uid: str
    :param uid: uid
    :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.RequestBodyApiAcquire.uid`

    :type enable_postpone_transcoding_mix: bool
    :param enable_postpone_transcoding_mix: 开启延时转码或延时混音

    :type resource_expired_hour: int
    :param resource_expired_hour: resource expired hour
    :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.ClientRequest.resourceExpiredHour`

    :type exclude_resource_ids: list
    :param exclude_resource_ids: exclude resource ids
    :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.ClientRequest.excludeResourceIds`

    :type region_affinity: int
    :param region_affinity: region affinity
    :refer: `agora_rest_client.services.cloud_recording.v1.api_acquire.ClientRequest.regionAffinity`
    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.RegionAffinity`

    :type trace_id: string
    :param trace_id: trace id

    :return: response object ResponseApiAcquire
    """
    request_body_obj = RequestBodyApiAcquire(
        cname=cname,
        uid=uid,
        clientRequest=ClientRequest(
            scene=Scene.INDIVIDUAL_RECORDING_POSTPONE_TRANSCODING_MIX.value if enable_postpone_transcoding_mix else Scene.RTC.value
    ))

    if resource_expired_hour is not None:
        request_body_obj.clientRequest.resourceExpiredHour = resource_expired_hour

    if exclude_resource_ids is not None:
        request_body_obj.clientRequest.excludeResourceIds = exclude_resource_ids

    if region_affinity is not None:
        request_body_obj.clientRequest.regionAffinity = region_affinity

    return api_acquire.api_acquire(client, request_body_obj=request_body_obj, response_obj=ResponseApiAcquire, trace_id=trace_id)



from agora_rest_client.core import request
from agora_rest_client.core import response

"""
官方文档: https://doc.shengwang.cn/api-ref/cloud-recording/restful/cloud-recording/operations/post-v1-apps-appid-cloud_recording-resourceid-resourceid-sid-sid-mode-mode-update
开始录制后, 你可以调用 update 方法更新如下录制配置:
对单流录制和合流录制, 更新订阅名单.
对页面录制, 设置暂停/恢复页面录制, 或更新页面录制转推到 CDN 的推流地址(URL).

对于每个声网账号, 每秒钟的请求数限制为 10 次. 如需提高此限制, 请联系技术支持.
update 请求仅在会话内有效. 如果录制启动错误, 或录制已结束, 调用 update 将返回 404.
如果需要连续调用 update 方法更新录制设置, 请在收到上一次 update 响应后再进行调用, 否则可能导致请求结果与预期不一致.
"""

class AudioUidList(request.RequestObject):
    """
    type: array[string]

    指定订阅哪几个 UID 的音频流. 如需订阅全部 UID 的音频流, 则无需设置该字段. 数组长度不得超过 32, 不推荐使用空数组. 该字段和 unsubscribeAudioUids 只能设一个. 详见设置订阅名单.

    注意:
    该字段仅适用于 streamTypes 设为音频, 或音频和视频的情况.
    如果你设置了音频的订阅名单, 但没有设置视频的订阅名单, 云端录制服务不会订阅任何视频流. 反之亦然.
    设为 ["#allstream#"]` 可订阅频道内所有 UID 的音频流.
    """
    subscribeAudioUids = None

    """
    type: array[string]

    指定不订阅哪几个 UID 的音频流. 云端录制会订阅频道内除指定 UID 外所有 UID 的音频流. 数组长度不得超过 32, 不推荐使用空数组. 该字段和 subscribeAudioUids 只能设一个. 详见设置订阅名单.
    """
    unsubscribeAudioUids = None

class VideoUidList(request.RequestObject):
    """
    type: array[string]

    指定订阅哪几个 UID 的视频流. 如需订阅全部 UID 的视频流, 则无需设置该字段. 数组长度不得超过 32, 不推荐使用空数组. 该字段和 unsubscribeVideoUids 只能设一个. 详见设置订阅名单.

    注意:
    该字段仅适用于 streamTypes 设为视频, 或音频和视频的情况.
    如果你设置了视频的订阅名单, 但没有设置音频的订阅名单, 云端录制服务不会订阅任何音频流. 反之亦然.
    设为 ["#allstream#"] 可订阅频道内所有 UID 的视频流.
    """
    subscribeVideoUids = None

    """
    type: array[string]

    指定不订阅哪几个 UID 的视频流. 云端录制会订阅频道内除指定 UID 外所有 UID 的视频流. 数组长度不得超过 32, 不推荐使用空数组. 该字段和 subscribeVideoUids 只能设一个. 详见设置订阅名单.
    """
    unsunscribeVideoUids = None

class StreamSubscribe(request.RequestObject):
    """
    type: object

    音频订阅名单.
    注意: 该字段仅适用于 streamTypes 设为音频, 或音频和视频的情况.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update.AudioUidList`
    """
    audioUidList = None

    """
    type: object

    视频订阅名单.
    注意: 该字段仅适用于 streamTypes 设为视频, 或音频和视频的情况.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update.VideoUidList`
    """
    videoUidList = None

class WebRecordingConfig(request.RequestObject):
    """
    type: boolean

    是否在启动页面录制任务时暂停页面录制.
    true: 在启动页面录制任务时暂停页面录制. 开启页面录制任务后立即暂停录制, 录制服务会打开并渲染待录制页面, 但不生成切片文件.
    false: 启动页面录制任务并进行页面录制.

    建议你按照如下流程使用 onhold 字段:
    调用 start 方法时将 onhold 设为 true, 开启并暂停页面录制, 自行判断页面录制开始的合适时机.
    调用 update 并将 onhold 设为 false, 继续进行页面录制. 如果需要连续调用 update 方法暂停或继续页面录制, 请在收到上一次 update 响应后再进行调用, 否则可能导致请求结果与预期不一致.

    default: false
    """
    onhold = None

class Outputs(request.RequestObject):
    """
    type: string

    CDN 推流 URL.

    注意:
    URL 仅支持 RTMP 和 RTMPS 协议.
    支持的最大转推 CDN 路数为 1.
    """
    rtmpUrl = None

class RtmpPublishConfig(request.RequestObject):
    """
    type: array[object]

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update.Outputs`
    """
    outputs = None

class ClientRequest(request.RequestObject):
    """
    type: object

    更新订阅名单.
    注意: 仅需在单流录制和合流录制模式下设置.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update.StreamSubscribe`
    """
    streamSubscribe = None

    """
    type: object

    用于更新页面录制配置项.
    注意: 仅需在页面录制模式下设置.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update.WebRecordingConfig`
    """
    webRecordingConfig = None

    """
    type: object

    用于更新转推页面录制到 CDN 的配置项.
    注意: 仅需在页面录制模式下, 且转推页面录制到 CDN 时设置.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update.RtmpPublishConfig`
    """
    rtmpPublishConfig = None

class RequestPathParamsApiUpdate(request.RequestObject):
    """
    type: required string

    录制模式:
    individual: 单流录制模式.
    mix: 合流录制模式.
    web: 页面录制模式.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.Mode`
    """
    mode = None

    """
    type: required string

    通过 acquire 请求获取到的 Resource ID.
    """
    resource_id = None

    """
    type: required string

    通过 start 获取的录制 ID.
    """
    sid = None

class RequestBodyApiUpdate(request.RequestObject):
    """
    type: required string

    录制服务所在频道的名称. 需要和你在 acquire 请求中输入的 cname 相同.
    """
    cname = None

    """
    type: required string

    字符串内容为录制服务在 RTC 频道内使用的 UID, 用于标识该录制服务, 需要和你在 acquire 请求中输入的 uid 相同.
    """
    uid = None

    """
    type: required object

    :value: instance `agora_rest_client.services.cloud_recording.v1.api_update.ClientRequest`
    """
    clientRequest = None

class ResponseApiUpdate(response.ResponseObject):
    """
    type: string

    录制的频道名.
    """
    cname = None

    """
    type: string

    字符串内容为云端录制服务在 RTC 频道内使用的 UID, 用于标识频道内的录制服务.
    """
    uid = None

    """
    type: string

    云端录制使用的 Resource ID.
    """
    resourceId = None

    """
    type: string

    录制 ID. 标识每次录制周期.
    """
    sid = None

def api_update(client, request_path_params_obj, request_body_obj, response_obj=ResponseApiUpdate, trace_id=None):
    """
    Update recording
    更新云端录制设置

    :type client: object
    :param client: CloudRecordingClient object

    :type request_path_params_obj: object
    :param request_path_params_obj: request path params object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update.RequestPathParamsApiUpdate`

    :type request_body_obj: object
    :param request_body_obj: request body object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update.RequestBodyApiUpdate`

    :type response_obj: object
    :param response_obj: response object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update.ResponseApiUpdate`

    :type trace_id: string
    :param trace_id: trace id

    :return: response object ResponseApiUpdate
    """
    url = '/v1/apps/{}/cloud_recording/resourceid/{}/sid/{}/mode/{}/update'.format(client.app_id, request_path_params_obj.resource_id, request_path_params_obj.sid, request_path_params_obj.mode)
    client.logger.debug("url:%s", url)

    return client.call_api('POST', url, post_json=request_body_obj.to_dict(), response_obj=response_obj, trace_id=trace_id)



from agora_rest_client.core import request
from agora_rest_client.core import response

"""
官方文档: https://doc.shengwang.cn/api-ref/cloud-recording/restful/cloud-recording/operations/post-v1-apps-appid-cloud_recording-resourceid-resourceid-sid-sid-mode-mode-updateLayout
开始合流录制后, 你可以调用该方法(updateLayout)更新合流布局.
每次调用该方法都会覆盖原来的布局设置. 例如, 在开始录制时设置了 backgroundColor 为 "#FF0000"(红色), 如果调用 updateLayout 方法更新合流布局时如果不再设置 backgroundColor 字段, 背景色就会变为黑色(默认值).

对于每个声网账号, 每秒钟的请求数限制为 10 次. 如需提高此限制, 请联系技术支持.
updateLayout 请求仅在会话内有效. 如果录制启动错误, 或录制已结束, 调用 updateLayout 将返回 404.
如果需要连续调用 updateLayout 方法更新合流布局, 请在收到上一次 updateLayout 响应后再进行调用, 否则可能导致请求结果与预期不一致.
"""

class LayoutConfig(request.RequestObject):
    """
    type: string

    字符串内容为待显示在该区域的用户的 UID, 32 位无符号整数.
    如果不指定 UID, 会按照用户加入频道的顺序自动匹配 layoutConfig 中的画面设置.
    """
    uid = None

    """
    type: required number<float>

    屏幕里该画面左上角的横坐标的相对值, 精确到小数点后六位. 从左到右布局, 0.0 在最左端, 1.0 在最右端. 该字段也可以设置为整数 0 或 1.
    >= 0 <= 1
    """
    x_axis = None

    """
    type: required number<float>

    屏幕里该画面左上角的纵坐标的相对值, 精确到小数点后六位. 从上到下布局, 0.0 在最上端, 1.0 在最下端. 该字段也可以设置为整数 0 或 1.
    >= 0 <= 1
    """
    y_axis = None

    """
    type: required number<float>

    该画面宽度的相对值, 精确到小数点后六位. 该字段也可以设置为整数 0 或 1.
    >= 0 <= 1
    """
    width = None

    """
    type: required number<float>

    该画面高度的相对值, 精确到小数点后六位. 该字段也可以设置为整数 0 或 1.
    >= 0 <= 1
    """
    height = None

    """
    type: number<float>

    图像的透明度. 精确到小数点后六位. 0.0 表示图像为透明的, 1.0 表示图像为完全不透明的.
    >= 0 <= 1

    default: 1
    """
    alpha = None

    """
    type: number

    画面显示模式:
    0: 裁剪模式. 优先保证画面被填满. 视频尺寸等比缩放, 直至整个画面被视频填满. 如果视频长宽与显示窗口不同, 则视频流会按照画面设置的比例进行周边裁剪后填满画面.
    1: 缩放模式. 优先保证视频内容全部显示. 视频尺寸等比缩放, 直至视频窗口的一边与画面边框对齐. 如果视频尺寸与画面尺寸不一致, 在保持长宽比的前提下, 将视频进行缩放后填满画面, 缩放后的视频四周会有一圈黑边.

    default: 0

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.RenderMode`
    """
    render_mode = None

class BackgroundConfig(request.RequestObject):
    """
    type: required string

    字符串内容为用户 UID.
    """
    uid = None

    """
    type: required string

    该用户的背景图 URL. 配置背景图后, 当该⽤户停止发送视频流超过 3.5 秒, 画⾯将切换为该背景图.
    URL 支持 HTTP 和 HTTPS 协议, 图片格式支持 JPG 和 BMP. 图片大小不得超过 6 MB. 录制服务成功下载图片后, 设置才会生效; 如果下载失败, 则设置不⽣效. 不同字段设置可能会互相覆盖, 具体规则详见设置背景色或背景图.
    """
    image_url = None

    """
    type: number

    画面显示模式:
    0: 裁剪模式. 优先保证画面被填满. 视频尺寸等比缩放, 直至整个画面被视频填满. 如果视频长宽与显示窗口不同, 则视频流会按照画面设置的比例进行周边裁剪后填满画面.
    1: 缩放模式. 优先保证视频内容全部显示. 视频尺寸等比缩放, 直至视频窗口的一边与画面边框对齐. 如果视频尺寸与画面尺寸不一致, 在保持长宽比的前提下, 将视频进行缩放后填满画面, 缩放后的视频四周会有一圈黑边.

    default: 0

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.RenderMode`
    """
    render_mode = None

class ClientRequest(request.RequestObject):
    """
    type: string

    仅需在垂直布局下设置. 指定显示大视窗画面的用户 UID. 字符串内容的整型取值范围 1 到 (232-1), 且不可设置为 0.
    """
    maxResolutionUid = None

    """
    type: number

    视频合流布局:
    0: 悬浮布局. 第一个加入频道的用户在屏幕上会显示为大视窗, 铺满整个画布, 其他用户的视频画面会显示为小视窗, 从下到上水平排列, 最多 4 行, 每行 4 个画面, 最多支持共 17 个画面.
    1: 自适应布局. 根据用户的数量自动调整每个画面的大小, 每个用户的画面大小一致, 最多支持 17 个画面.
    2: 垂直布局. 指定 maxResolutionUid 在屏幕左侧显示大视窗画面, 其他用户的小视窗画面在右侧垂直排列, 最多两列, 一列 8 个画面, 最多支持共 17 个画面.
    3: 自定义布局. 由你在 layoutConfig 字段中自定义合流布局.

    default: 0

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.MixedVideoLayout`
    """
    mixedVideoLayout = None

    """
    type: string

    视频画布的背景颜色. 支持 RGB 颜色表, 字符串格式为 # 号和 6 个十六进制数. 默认值 "#000000", 代表黑色.

    default: #000000
    """
    backgroundColor = None

    """
    type: string

    视频画布的背景图的 URL. 背景图的显示模式为裁剪模式.
    裁剪模式: 优先保证画面被填满. 背景图尺寸等比缩放, 直至整个画面被背景图填满. 如果背景图长宽与显示窗口不同, 则背景图会按照画面设置的比例进行周边裁剪后填满画面.
    """
    backgroundImage = None

    """
    type: string

    默认的用户画面背景图的 URL.
    配置该字段后, 当任一⽤户停止发送视频流超过 3.5 秒, 画⾯将切换为该背景图; 如果针对某 UID 单独设置了背景图, 则该设置会被覆盖.
    """
    defaultUserBackgroundImage = None

    """
    type: array[object]

    用户的合流画面布局. 由每个用户对应的布局画面设置组成的数组, 支持最多 17 个用户.
    注意: 仅需在自定义布局下设置.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update_layout.LayoutConfig`
    """
    layoutConfig = None

    """
    type: array[object]

    用户的背景图设置.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update_layout.BackgroundConfig`
    """
    backgroundConfig = None

class RequestPathParamsApiUpdateLayout(request.RequestObject):
    """
    type: required string

    录制模式. 只支持 mix, 代表合流录制模式.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.Mode`
    """
    mode = None

    """
    type: required string

    通过 acquire 请求获取到的 Resource ID.
    """
    resource_id = None

    """
    type: required string

    通过 start 获取的录制 ID.
    """
    sid = None

class RequestBodyApiUpdateLayout(request.RequestObject):
    """
    type: required string

    录制服务所在频道的名称. 需要和你在 acquire 请求中输入的 cname 相同.
    """
    cname = None

    """
    type: required string

    字符串内容为录制服务在 RTC 频道内使用的 UID, 用于标识该录制服务, 需要和你在 acquire 请求中输入的 uid 相同.
    """
    uid = None

    """
    type: required object

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest`
    """
    clientRequest = None

class ResponseApiUpdateLayout(response.ResponseObject):
    """
    type: string

    录制的频道名.
    """
    cname = None

    """
    type: string

    字符串内容为云端录制服务在 RTC 频道内使用的 UID, 用于标识频道内的录制服务.
    """
    uid = None

    """
    type: string

    云端录制使用的 Resource ID.
    """
    resourceId = None

    """
    type: string

    录制 ID. 标识一次录制周期.
    """
    sid = None

def api_update_layout(client, request_path_params_obj, request_body_obj, response_obj=ResponseApiUpdateLayout, trace_id=None):
    """
    Update layout of the recording
    更新云端录制合流布局

    :type client: object
    :param client: CloudRecordingClient object

    :type request_path_params_obj: object
    :param request_path_params_obj: request path params object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update_layout.RequestPathParamsApiUpdateLayout`

    :type request_body_obj: object
    :param request_body_obj: request body object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update_layout.RequestBodyApiUpdateLayout`

    :type response_obj: object
    :param response_obj: response object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_update_layout.ResponseApiUpdateLayout`

    :type trace_id: string
    :param trace_id: trace id

    :return: response object ResponseApiUpdateLayout
    """
    url = '/v1/apps/{}/cloud_recording/resourceid/{}/sid/{}/mode/{}/updateLayout'.format(client.app_id, request_path_params_obj.resource_id, request_path_params_obj.sid, request_path_params_obj.mode)
    client.logger.debug("url:%s", url)

    return client.call_api('POST', url, post_json=request_body_obj.to_dict(), response_obj=response_obj, trace_id=trace_id)



from agora_rest_client.services.cloud_recording.v1 import api_query
from agora_rest_client.services.cloud_recording.v1.api import Mode

class RequestPathParamsApiQuery(api_query.RequestPathParamsApiQuery):
    pass

class ResponseApiQuery(api_query.ResponseApiQuery):
    pass

def web_recording_query(client, resource_id, sid, trace_id=None):
    """
    Web recording query
    查询云端录制状态

    :type client: object
    :param client: WebRecordingClient object

    :type resource_id: str
    :param resource_id: resource id
    :refer: `agora_rest_client.services.cloud_recording.v1.api_query.RequestPathParamsApiQuery.resource_id`

    :type sid: str
    :param sid: sid
    :refer: `agora_rest_client.services.cloud_recording.v1.api_query.RequestPathParamsApiQuery.sid`

    :type trace_id: string
    :param trace_id: trace id

    :return: response object ResponseApiQuery
    """
    request_path_params_obj = RequestPathParamsApiQuery(
        mode=Mode.WEB.value,
        resource_id=resource_id,
        sid=sid
    )

    return api_query.api_query(client, request_path_params_obj=request_path_params_obj, response_obj=ResponseApiQuery, trace_id=trace_id)



import func_timeout
import logging
import requests
import sys
import time
from logging.handlers import RotatingFileHandler
from agora_rest_client.core import domain
from agora_rest_client.core import errors
from agora_rest_client.core import exceptions
from agora_rest_client.core import log

class Client(object):
    """
    Base client
    """

    def __init__(self):
        self._app_id = None
        self._basic_auth = None
        self._domain = domain.default_domain
        self._file_logger_handler = None
        self._http_retry_count = 3
        self._http_timeout_seconds = 10
        self._logger = self._init_logger()
        self._stream_logger_handler = None

    @classmethod
    def _init_logger(cls):
        """
        Init logger
        """
        logger_name = '%s-%s-%d' % (log.LOGGER_NAME, cls.__name__, time.time()*1000000)
        logger = logging.getLogger(logger_name)
        logger.propagate = False

        return logger

    def new_builder():
        return Client()

    def build(self):
        """
        Build client
        """
        if self._app_id is None:
            raise exceptions.ClientBuildException(errors.APP_ID_REQUIRED)

        if self._basic_auth is None:
            raise exceptions.ClientBuildException(errors.BASIC_AUTH_REQUIRED)

        # Check region
        if self._domain.get_region() is None:
            raise exceptions.ClientBuildException(errors.REGION_REQUIRED)

        if self._domain.get_region() not in domain.RegionArea._value2member_map_:
            raise exceptions.ClientBuildException(errors.REGION_INVALID)

        if self._file_logger_handler is not None:
            self.add_file_logger(**self._file_logger_handler)
        if self._stream_logger_handler is not None:
            self.add_stream_logger(**self._stream_logger_handler)

        self._domain.run(self._logger)

        return self

    def add_file_logger(self, path, log_level, max_bytes, backup_count, format_string):
        """
        Add file logger

        :type path: str
        :param path: log file path

        :type log_level: int
        :param log_level: log level

        :type max_bytes: int
        :param max_bytes: max bytes

        :type backup_count: int
        :param backup_count: backup count

        :type format_string: str
        :param format_string: format string
        """
        self._logger.setLevel(log_level)
        file_handler = RotatingFileHandler(path, maxBytes=max_bytes, backupCount=backup_count)
        file_handler.setLevel(log_level)
        formatter = logging.Formatter(format_string if format_string is not None else log.LOGGER_FORMAT)
        file_handler.setFormatter(formatter)

        if file_handler not in self._logger.handlers:
            self._logger.addHandler(file_handler)

    def add_stream_logger(self, stream, log_level, format_string):
        """
        Add stream logger

        :type stream: object
        :param stream: stream

        :type log_level: int
        :param log_level: log level

        :type format_string: str
        :param format_string: format string
        """
        self._logger.setLevel(log_level)
        stream_handler = logging.StreamHandler(stream)
        stream_handler.setLevel(log_level)
        formatter = logging.Formatter(format_string if format_string is not None else log.LOGGER_FORMAT)
        stream_handler.setFormatter(formatter)

        if stream_handler not in self._logger.handlers:
            self._logger.addHandler(stream_handler)

    @property
    def app_id(self):
        return self._app_id

    def call_api(self, method, url, params=None, post_data=None, post_json=None, headers=None, timeout_seconds=5, trace_id=None):
        """
        Call api

        :type method: str
        :param method: http method

        :type url: str
        :param url: http url

        :type params: object
        :param params: http params

        :type post_data: object
        :param post_data: http post data

        :type post_json: object
        :param post_json: http post json

        :type headers: object
        :param headers: http headers

        :type timeout_seconds: int
        :param timeout_seconds: http timeout

        :type trace_id: string
        :param trace_id: trace id

        :type: object
        :return: class:`requests.Response <Response>` object
        """
        try:
            resp = func_timeout.func_timeout(self._http_timeout_seconds, self.do_http_request, args=(method, url),
                                             kwargs={'params': params, 'post_data': post_data, 'post_json': post_json, 'headers': headers,
                                                     'timeout_seconds': timeout_seconds, 'trace_id': trace_id})
            return resp
        except func_timeout.FunctionTimedOut as e:
            raise exceptions.ClientTimeoutException(None, None, e.getMsg())
        except Exception as e:
            raise exceptions.ClientRequestException(None, None, str(e))

    def do_http_request(self, method, url, params=None, post_data=None, post_json=None, headers=None, timeout_seconds=5, trace_id=None):
        """
        Request http

        :type method: str
        :param method: http method

        :type url: str
        :param url: http url

        :type params: object
        :param params: http params

        :type post_data: object
        :param post_data: http post data

        :type post_json: object
        :param post_json: http post json

        :type headers: object
        :param headers: http headers

        :type timeout_seconds: int
        :param timeout_seconds: http timeout

        :type trace_id: string
        :param trace_id: trace id

        :type: object
        :return: class:`requests.Response <Response>` object
        """
        status_code = None

        while True:
            for host in self._domain.get_domain_list():
                host_url = 'https://%s%s' % (host, url)
                self._logger.debug('do http request, trace_id:%s, host_url:%s', trace_id, host_url)

                try:
                    resp = requests.request(method, host_url, params=params, data=post_data, json=post_json, headers=headers,
                                            timeout=timeout_seconds, auth=self._basic_auth)
                    status_code = resp.status_code

                    self._logger.debug('do http request, trace_id:%s, host_url:%s, status_code:%s', trace_id, host_url, status_code)
                    return resp
                except requests.exceptions.RequestException as e:
                    self._logger.error('do http request, request failed, err:%s, trace_id:%s, host_url:%s, status_code:%s', e, trace_id, host_url, status_code)

                # Sleep
                time.sleep(0.5)

    @property
    def logger(self):
        return self._logger

    def with_app_id(self, app_id):
        self._app_id = app_id

        return self

    def with_basic_auth(self, user_name, password):
        self._basic_auth = (user_name, password)

        return self

    def with_http_retry_count(self, _http_retry_count):
        self._http_retry_count = _http_retry_count

        return self

    def with_http_timeout_seconds(self, http_timeout_seconds):
        self._http_timeout_seconds = http_timeout_seconds

        return self

    def with_file_log(self, path, log_level=logging.INFO, max_bytes=10 * 1024 * 1024, backup_count=2, format_string=None):
        self._file_logger_handler = {
            "backup_count": backup_count,
            "format_string": format_string,
            "log_level": log_level,
            "max_bytes": max_bytes,
            "path": path
        }

        return self

    def with_region(self, region):
        self._domain.set_region(region)

        return self

    def with_stream_log(self, stream=sys.stdout, log_level=logging.INFO, format_string=None):
        self._stream_logger_handler = {
            "format_string": format_string,
            "log_level": log_level,
            "stream": stream
        }

        return self



from agora_rest_client.core import request
from agora_rest_client.core import response

"""
官方文档: https://doc.shengwang.cn/api-ref/cloud-recording/restful/cloud-recording/operations/post-v1-apps-appid-cloud_recording-acquire
在开始云端录制之前, 你需要调用 acquire 方法获取一个 Resource ID. 一个 Resource ID 只能用于一次云端录制服务.
为确保成功开始云端录制, 请在每次 acquire 请求获取到 Resource ID 后的 2 秒内立即发起对应的 start 请求.
批量获取 Resource ID 后进行批量 start 请求可能导致请求失败. acquire 和 start 的请求需配对调用. Resource ID 在获取到的 5 分钟内有效, 需尽快使用.
对于每个声网账号，每秒钟的请求数限制为 10 次。如需提高此限制，请联系技术支持
"""

class ClientRequest(request.RequestObject):
    """
    type: number

    云端录制资源使用场景:
    0: 实时音视频录制.
    1: 页面录制.
    2: 单流录制模式, 且开启延时转码或延时混音.
        延时转码: 录制服务会在录制后 24 小时内(特殊情况下会到 48 小时以上)对录制文件进行转码生成 MP4 文件, 并将 MP4 文件上传至你指定的第三方云存储. 该场景仅适用于单流录制模式.
        延时混音: 录制服务会在录制结束后 24 小时内(特殊情况下会到 48 小时以上)将指定频道内所有 UID 的录制文件合并并转码生成一个 MP3/M4A/AAC 文件, 并将文件上传至你指定的第三方云存储. 该场景仅适用于音频单流不转码录制模式.
        scene 设为 2 时, 你需要同时在 start 方法中设置 appsCollection 字段.
        在延时转码和延时混音的场景下, 录制文件会在声网边缘服务器上缓存, 最长不超过 24 小时. 如果你的业务对信息安全敏感, 为了确保数据合规, 请慎重考虑是否使用延时转码和延时混音功能. 如有任何疑虑, 请联系声网技术支持.
        在延时转码和延时混音的场景中, 声网建议你不要填写 region 字段, 也不要将其留空. 否则, 由于声网边缘服务器的动态调整和延时场景下录制文件缓存带来的数据合规风险, 声网录制服务将无法正常使用.

    default: 0

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.Scene`
    """
    scene = None

    """
    type: number

    云端录制 RESTful API 的调用时效. 从成功开启云端录制并获得 sid (录制 ID)后开始计算. 单位为小时.
    注意: 超时后, 你将无法调用 query, update, updateLayout 和 stop 方法.
    >= 1 <= 720

    default: 72
    """
    resourceExpiredHour = None

    """
    type: array[string]

    另一路或几路录制任务的 resourceId. 该字段用于排除指定的录制资源, 以便新发起的录制任务可以使用新区域的资源, 实现跨区域多路录制
    """
    excludeResourceIds = None

    """
    type: number

    指定使用某个区域的资源进行录制. 支持取值如下:
    0: 根据发起请求的区域就近调用资源.
    1: 中国.
    2: 东南亚.
    3: 欧洲.
    4: 北美.
    注意: 为加速录制文件上传, 当你使用的云存储区域和你发起请求的区域不同时, 建议你将该字段设为云存储区域.

    default: 0

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.RegionAffinity`
    """
    regionAffinity = None

class RequestBodyApiAcquire(request.RequestObject):
    """
    type: required string

    频道名:
    对于单流录制和合流录制模式, 该字段用于设置待录制的频道名.
    对于页面录制模式, 该字段用于区分录制进程. 字符串长度不得超过 128 字节.
    注意: 通过 appid/cname 和 uid 可以定位一个唯一的录制实例. 因此, 如果你想针对同一个频道进行多次录制, 可以使用相同的 appId 和 cname, 以及不同的 uid 来进行管理和区分.
    """
    cname = None

    """
    type: required string

    字符串内容为云端录制服务在频道内使用的 UID, 用于标识频道内的录制服务, 例如 "527841". 字符串内容需满足以下条件:
    取值范围 1 到 (232-1), 不可设置为 0.
    不能与当前频道内的任何 UID 重复.
    字段引号内为整型 UID, 且频道内所有用户均使用整型 UID.
    """
    uid = None

    """
    type: object

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_acquire.ClientRequest`
    """
    clientRequest = None

class ResponseApiAcquire(response.ResponseObject):
    """
    type: string

    录制的频道名.
    """
    cname = None

    """
    type: string

    字符串内容为云端录制服务在 RTC 频道内使用的 UID, 用于标识频道内的录制服务.
    """
    uid = None

    """
    type: string

    云端录制资源 Resource ID. 使用这个 Resource ID 可以开始一段云端录制. 这个 Resource ID 的有效期为 5 分钟, 超时需要重新请求.
    """
    resourceId = None

def api_acquire(client, request_body_obj, response_obj=ResponseApiAcquire, trace_id=None):
    """
    Acquire a resource id
    获取云端录制资源

    :type client: object
    :param client: CloudRecordingClient object

    :type request_body_obj: object
    :param request_body_obj: request body object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_acquire.RequestBodyApiAcquire`

    :type response_obj: object
    :param response_obj: response object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_acquire.ResponseApiAcquire`

    :type trace_id: string
    :param trace_id: trace id

    :return: response object ResponseApiAcquire
    """
    url = '/v1/apps/{}/cloud_recording/acquire'.format(client.app_id)
    client.logger.debug("url:%s", url)

    return client.call_api('POST', url, post_json=request_body_obj.to_dict(), response_obj=response_obj, trace_id=trace_id)



# coding: utf-8
from os import path
from setuptools import setup, find_packages

NAME = "agora_rest_client"
VERSION = "0.1.0"
AUTHOR = "Agora REST Client"
URL = "https://github.com/AgoraIO-Community/agora-rest-client-python"

DESCRIPTION = "agora rest client"
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README_PYPI.md'), encoding='utf-8') as f:
    LONG_DESCRIPTION = f.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

OPTIONS = {
    'bdist_wheel': {
        'universal': True
    }
}

setup(
    name=NAME,
    version=VERSION,
    options=OPTIONS,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    license="Apache LICENSE 2.0",
    url=URL,
    keywords=["agora", "rest", "client", "agora_rest_client", "agora-rest-client"],
    packages=find_packages(exclude=["tests*"]),
    install_requires=requirements,
    python_requires="!=3.0.*,!=3.1.*,!=3.2.*",
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development'
    ]
)



from agora_rest_client.core import request
from agora_rest_client.core import response

"""
官方文档: https://doc.shengwang.cn/api-ref/cloud-recording/restful/cloud-recording/operations/post-v1-apps-appid-cloud_recording-resourceid-resourceid-mode-mode-start
通过 acquire 方法获取云端录制资源后, 调用 start 方法开始云端录制.
发起 start 请求后, 你还需要依据最佳实践检查录制服务是否已经成功启动.
对于每个声网账号, 每秒钟的请求数限制为 10 次. 如需提高此限制, 请联系技术支持
"""

class ExtensionParams(request.RequestObject):
    """
    type: required string

    加密模式. 设置该字段后, 第三方云存储服务会按照该加密模式将已上传的录制文件进行加密. 该字段仅适用于 Amazon S3
    kms: KMS 加密.
    aes256: AES256 加密.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.AmazonS3Sse`
    """
    sse = None

    """
    type: required string

    标签内容. 设置该字段后, 第三方云存储服务会按照该标签内容将已上传的录制文件进行打标签操作. 该字段仅适用于阿里云和 Amazon S3
    """
    tag = None

class StorageConfig(request.RequestObject):
    """
    type: required number

    第三方云存储指定的地区信息.
    注意: 为确保录制文件上传的成功率和实时性, 第三方云存储的 region 与你发起请求的应用服务器必须在同一个区域中. 例如: 你发起请求的 App 服务器在中国大陆地区, 则第三方云存储需要设置为中国大陆区域内
    """
    region = None

    """
    type: required number

    第三方云存储平台.
    1: Amazon S3
    2: 阿里云
    3: 腾讯云
    5: Microsoft Azure
    6: 谷歌云
    7: 华为云
    8: 百度智能云

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.StorageVendor`
    """
    vendor = None

    """
    type: required string

    第三方云存储的 Bucket. Bucket 名称需要符合对应第三方云存储服务的命名规则.
    """
    bucket = None

    """
    type: required string

    第三方云存储的 Access Key(访问密钥). 如需延时转码, 则访问密钥必须具备读写权限; 否则建议只需提供写权限.
    """
    accessKey = None

    """
    type: required string

    第三方云存储的 Secret Key.
    """
    secretKey = None

    """
    type: array[string]

    录制文件在第三方云存储中的存储位置, 与录制文件名前缀有关. 如果设为 ["directory1","directory2"], 那么录制文件名前缀为 "directory1/directory2/", 即录制文件名为 directory1/directory2/xxx.m3u8. 前缀长度(包括斜杠)不得超过 128 个字符. 字符串中不得出现斜杠/下划线/括号等符号字符. 以下为支持的字符集范围:
    26 个小写英文字母 a~z
    26 个大写英文字母 A~Z
    10 个数字 0-9
    """
    fileNamePrefix = None

    """
    type: object

    第三方云存储服务会按照该字段设置对已上传的录制文件进行加密和打标签.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.ExtensionParams`
    """
    extensionParams = None

class LayoutConfig(request.RequestObject):
    """
    type: string

    字符串内容为待显示在该区域的用户的 UID, 32 位无符号整数.
    如果不指定 UID, 会按照用户加入频道的顺序自动匹配 layoutConfig 中的画面设置.
    """
    uid = None

    """
    type: required number<float>

    屏幕里该画面左上角的横坐标的相对值, 精确到小数点后六位. 从左到右布局, 0.0 在最左端, 1.0 在最右端. 该字段也可以设置为整数 0 或 1.
    >= 0 <= 1
    """
    x_axis = None

    """
    type: required number<float>

    屏幕里该画面左上角的纵坐标的相对值, 精确到小数点后六位. 从上到下布局, 0.0 在最上端, 1.0 在最下端. 该字段也可以设置为整数 0 或 1.
    >= 0 <= 1
    """
    y_axis = None

    """
    type: required number<float>

    该画面宽度的相对值, 精确到小数点后六位. 该字段也可以设置为整数 0 或 1.
    >= 0 <= 1
    """
    width = None

    """
    type: required number<float>

    该画面高度的相对值, 精确到小数点后六位. 该字段也可以设置为整数 0 或 1.
    >= 0 <= 1
    """
    height = None

    """
    type: number<float>

    图像的透明度. 精确到小数点后六位. 0.0 表示图像为透明的, 1.0 表示图像为完全不透明的.
    >= 0 <= 1

    default: 1
    """
    alpha = None

    """
    type: number

    画面显示模式:
    0: 裁剪模式. 优先保证画面被填满. 视频尺寸等比缩放, 直至整个画面被视频填满. 如果视频长宽与显示窗口不同, 则视频流会按照画面设置的比例进行周边裁剪后填满画面.
    1: 缩放模式. 优先保证视频内容全部显示. 视频尺寸等比缩放, 直至视频窗口的一边与画面边框对齐. 如果视频尺寸与画面尺寸不一致, 在保持长宽比的前提下, 将视频进行缩放后填满画面, 缩放后的视频四周会有一圈黑边.

    default: 0

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.RenderMode`
    """
    render_mode = None

class BackgroundConfig(request.RequestObject):
    """
    type: required string

    字符串内容为用户 UID.
    """
    uid = None

    """
    type: required string

    该用户的背景图 URL. 配置背景图后, 当该⽤户停止发送视频流超过 3.5 秒, 画⾯将切换为该背景图.
    URL 支持 HTTP 和 HTTPS 协议, 图片格式支持 JPG 和 BMP. 图片大小不得超过 6 MB. 录制服务成功下载图片后, 设置才会生效; 如果下载失败, 则设置不⽣效. 不同字段设置可能会互相覆盖, 具体规则详见设置背景色或背景图.
    """
    image_url = None

    """
    type: number

    画面显示模式:
    0: 裁剪模式. 优先保证画面被填满. 视频尺寸等比缩放, 直至整个画面被视频填满. 如果视频长宽与显示窗口不同, 则视频流会按照画面设置的比例进行周边裁剪后填满画面.
    1: 缩放模式. 优先保证视频内容全部显示. 视频尺寸等比缩放, 直至视频窗口的一边与画面边框对齐. 如果视频尺寸与画面尺寸不一致, 在保持长宽比的前提下, 将视频进行缩放后填满画面, 缩放后的视频四周会有一圈黑边.

    default: 0

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.RenderMode`
    """
    render_mode = None

class TranscodingConfig(request.RequestObject):
    """
    type: required number

    视频的宽度, 单位为像素. width 和 height 的乘积不能超过 1920 × 1080.
    <= 1920

    default: 360
    """
    width = None

    """
    type: required number

    视频的高度, 单位为像素. width 和 height 的乘积不能超过 1920 × 1080.
    <= 1920

    default: 640
    """
    height = None

    """
    type: required number

    视频的帧率, 单位 fps.

    default: 15
    """
    fps = None

    """
    type: required number

    视频的码率, 单位 Kbps.

    default: 500
    """
    bitrate = None

    """
    type: string

    仅需在垂直布局下设置. 指定显示大视窗画面的用户 UID. 字符串内容的整型取值范围 1 到 (232-1), 且不可设置为 0.
    """
    maxResolutionUid = None

    """
    type: number

    视频合流布局:
    0: 悬浮布局. 第一个加入频道的用户在屏幕上会显示为大视窗, 铺满整个画布, 其他用户的视频画面会显示为小视窗, 从下到上水平排列, 最多 4 行, 每行 4 个画面, 最多支持共 17 个画面.
    1: 自适应布局. 根据用户的数量自动调整每个画面的大小, 每个用户的画面大小一致, 最多支持 17 个画面.
    2: 垂直布局. 指定 maxResolutionUid 在屏幕左侧显示大视窗画面, 其他用户的小视窗画面在右侧垂直排列, 最多两列, 一列 8 个画面, 最多支持共 17 个画面.
    3: 自定义布局. 由你在 layoutConfig 字段中自定义合流布局.

    default: 0

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.MixedVideoLayout`
    """
    mixedVideoLayout = None

    """
    type: string

    视频画布的背景颜色. 支持 RGB 颜色表, 字符串格式为 # 号和 6 个十六进制数. 默认值 "#000000", 代表黑色.

    default: #000000
    """
    backgroundColor = None

    """
    type: string

    视频画布的背景图的 URL. 背景图的显示模式为裁剪模式.
    裁剪模式: 优先保证画面被填满. 背景图尺寸等比缩放, 直至整个画面被背景图填满. 如果背景图长宽与显示窗口不同, 则背景图会按照画面设置的比例进行周边裁剪后填满画面.
    """
    backgroundImage = None

    """
    type: string

    默认的用户画面背景图的 URL.
    配置该字段后, 当任一⽤户停止发送视频流超过 3.5 秒, 画⾯将切换为该背景图; 如果针对某 UID 单独设置了背景图, 则该设置会被覆盖.
    """
    defaultUserBackgroundImage = None

    """
    type: array[object]

    用户的合流画面布局. 由每个用户对应的布局画面设置组成的数组, 支持最多 17 个用户.
    注意: 仅需在自定义布局下设置.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.LayoutConfig`
    """
    layoutConfig = None

    """
    type: array[object]

    用户的背景图设置.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.BackgroundConfig`
    """
    backgroundConfig = None

class RecordingConfig(request.RequestObject):
    """
    type: required number

    频道场景.
    0: 通信场景.
    1: 直播场景.
    频道场景必须与声网 RTC SDK 的设置一致, 否则可能导致问题.

    default: 0

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.ChannelType`
    """
    channelType = None

    """
    type: number

    解密模式. 如果你在 SDK 客户端设置了频道加密, 那么你需要对云端录制服务设置与加密相同的解密模式.
    0: 不加密.
    1: AES_128_XTS 加密模式. 128 位 AES 加密, XTS 模式.
    2: AES_128_ECB 加密模式. 128 位 AES 加密, ECB 模式.
    3: AES_256_XTS 加密模式. 256 位 AES 加密, XTS 模式.
    4: SM4_128_ECB 加密模式. 128 位 SM4 加密, ECB 模式.
    5: AES_128_GCM 加密模式. 128 位 AES 加密, GCM 模式.
    6: AES_256_GCM 加密模式. 256 位 AES 加密, GCM 模式.
    7: AES_128_GCM2 加密模式. 128 位 AES 加密, GCM 模式. 相比于 AES_128_GCM 加密模式, AES_128_GCM2 加密模式安全性更高且需要设置密钥和盐.
    8: AES_256_GCM2 加密模式. 256 位 AES 加密, GCM 模式. 相比于 AES_256_GCM 加密模式, AES_256_GCM2 加密模式安全性更高且需要设置密钥和盐.

    default: 0

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.DecryptionMode`
    """
    decryptionMode = None

    """
    type: string

    与加解密相关的密钥. 仅需在 decryptionMode 非 0 时设置.
    """
    secret = None

    """
    type: string

    与加解密相关的盐. Base64 编码/32 位字节. 仅需在 decryptionMode 为 7 或 8 时设置.
    """
    salt = None

    """
    type: number

    最大频道空闲时间. 单位为秒. 最大值不超过 30 天. 超出最大频道空闲时间后, 录制服务会自动退出. 录制服务退出后, 如果你再次发起 start 请求, 会产生新的录制文件.
    频道空闲: 直播频道内无任何主播, 或通信频道内无任何用户.
    >= 5 <= 2592000

    default: 30
    """
    maxIdleTime = None

    """
    type: number

    订阅的媒体流类型.
    0: 仅订阅音频. 适用于智能语音审核场景.
    1: 仅订阅视频.
    2: 订阅音频和视频.

    default: 2

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.StreamTypes`
    """
    streamTypes = None

    """
    type: number

    设置订阅的视频流类型. 如果你在 SDK 客户端开启了双流模式, 你可以选择订阅视频大流或者小流.
    0: 视频大流, 即高分辨率高码率的视频流
    1: 视频小流, 即低分辨率低码率的视频流

    default: 0

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.VideoStreamType`
    """
    videoStreamType = None

    """
    type: array[string]

    指定订阅哪几个 UID 的音频流. 如需订阅全部 UID 的音频流, 则无需设置该字段. 数组长度不得超过 32, 不推荐使用空数组. 该字段和 unsubscribeAudioUids 只能设一个. 详见设置订阅名单.
    注意:
    该字段仅适用于 streamTypes 设为音频, 或音频和视频的情况.
    如果你设置了音频的订阅名单, 但没有设置视频的订阅名单, 云端录制服务不会订阅任何视频流. 反之亦然.
    设为 ["#allstream#"]` 可订阅频道内所有 UID 的音频流.
    """
    subscribeAudioUids = None

    """
    type: array[string]

    指定不订阅哪几个 UID 的音频流. 云端录制会订阅频道内除指定 UID 外所有 UID 的音频流. 数组长度不得超过 32, 不推荐使用空数组. 该字段和 subscribeAudioUids 只能设一个. 详见设置订阅名单.
    """
    unsubscribeAudioUids = None

    """
    type: array[string]

    指定订阅哪几个 UID 的视频流. 如需订阅全部 UID 的视频流, 则无需设置该字段. 数组长度不得超过 32, 不推荐使用空数组. 该字段和 unsubscribeVideoUids 只能设一个. 详见设置订阅名单.
    注意:
    该字段仅适用于 streamTypes 设为视频, 或音频和视频的情况.
    如果你设置了视频的订阅名单, 但没有设置音频的订阅名单, 云端录制服务不会订阅任何音频流. 反之亦然.
    设为 ["#allstream#"] 可订阅频道内所有 UID 的视频流.
    """
    subscribeVideoUids = None

    """
    type: array[string]

    指定不订阅哪几个 UID 的视频流. 云端录制会订阅频道内除指定 UID 外所有 UID 的视频流. 数组长度不得超过 32, 不推荐使用空数组. 该字段和 subscribeVideoUids 只能设一个. 详见设置订阅名单.
    """
    unsubscribeVideoUids = None

    """
    type: number

    预估的订阅人数峰值.
    0: 1 到 2 个 UID.
    1: 3 到 7 个 UID.
    2: 8 到 12 个 UID.
    3: 13 到 17 个 UID.
    4: 17 到 32 个 UID.
    5: 32 到 49 个 UID.

    注意:
    仅需在单流录制模式下设置, 且单流录制模式下必填.
    举例来说, 如果 subscribeVideoUids 为 ["100","101","102"], subscribeAudioUids 为 ["101","102","103"], 则订阅人数为 4 人.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.SubscribeUidGroup`
    """
    subscribeUidGroup = None

    """
    type: string

    媒体流的输出模式. 详见媒体流输出模式.
    "default": 默认模式. 录制过程中音频转码, 分别生成 M3U8 音频索引文件和视频索引文件.
    "standard": 标准模式. 声网推荐使用该模式. 录制过程中音频转码, 分别生成 M3U8 音频索引文件/视频索引文件和合并的音视频索引文件. 如果在 Web 端使用 VP8 编码, 则生成一个合并的 MPD 音视频索引文件.
    "original": 原始编码模式. 适用于单流音频不转码录制. 仅订阅音频时(streamTypes 为 0)时该字段生效, 录制过程中音频不转码, 生成 M3U8 音频索引文件.

    注意: 仅需在单流录制模式下设置.

    default: default

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.StreamMode`
    """
    streamMode = None

    """
    type: number

    设置输出音频的采样率/码率/编码模式和声道数.
    0: 48 kHz 采样率, 音乐编码, 单声道, 编码码率约 48 Kbps.
    1: 48 kHz 采样率, 音乐编码, 单声道, 编码码率约 128 Kbps.
    2: 48 kHz 采样率, 音乐编码, 双声道, 编码码率约 192 Kbps.

    注意: 仅需在合流录制模式下设置.

    default: 0

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.AudioProfile`
    """
    audioProfile = None

    """
    type: object

    转码输出的视频配置项. 取值可参考设置录制输出视频的分辨率.

    注意: 仅需在单流录制和合流录制模式下设置.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.TranscodingConfig`
    """
    transcodingConfig = None

class RecordingFileConfig(request.RequestObject):
    """
    type: array[string]

    录制生成的视频文件类型:
    "hls": 默认值. M3U8 和 TS 文件.
    "mp4": MP4 文件.

    注意:
    单流录制模式下, 且非仅截图情况, 使用默认值即可.
    合流录制和页面录制模式下, 你需设为 ["hls","mp4"]. 仅设为 ["mp4"] 会收到报错. 设置后, 录制文件行为如下:
        合流录制模式: 录制服务会在当前 MP4 文件时长超过约 2 小时或文件大小超过约 2 GB 左右时, 创建一个新的 MP4 文件.
        页面录制模式: 录制服务会在当前 MP4 文件时长超过 maxVideoDuration 时, 创建一个新的 MP4 文件.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.AvFileType`
    """
    avFileType = None

class SnapshotConfig(request.RequestObject):
    """
    type: number

    云端录制定期截图的截图周期. 单位为秒.
    >= 5 <= 3600

    default: 10
    """
    captureInterval = None

    """
    type: required array[string]

    截图的文件格式. 目前只支持 ["jpg"], 即生成 JPG 格式的截图文件.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.SnapshotFileType`
    """
    fileType = None

class ServiceParam(request.RequestObject):
    """
    type: required string

    待录制页面的地址.
    """
    url = None

    """
    type: required number

    输出音频的采样率/码率/编码模式和声道数.
    0: 48 kHz 采样率, 音乐编码, 单声道, 编码码率约 48 Kbps.
    1: 48 kHz 采样率, 音乐编码, 单声道, 编码码率约 128 Kbps.
    2: 48 kHz 采样率, 音乐编码, 双声道, 编码码率约 192 Kbps.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.AudioProfile`
    """
    audioProfile = None

    """
    type: required number

    输出视频的宽度, 单位为 pixel. videoWidth 和 videoHeight 的乘积需小于等于 1920 × 1080. 推荐值可参考如何设置页面录制移动端网页模式的输出视频分辨率.
    >= 240 <= 1920
    """
    videoWidth = None

    """
    type: required number

    输出视频的高度, 单位为 pixel. videoWidth 和 videoHeight 的乘积需小于等于 1920 × 1080. 推荐值可参考如何设置页面录制移动端网页模式的输出视频分辨率.
    >= 240 <= 1920
    """
    videoHeight = None

    """
    type: required number

    页面录制的最大时长, 单位为小时. 超出该值后页面录制会自动停止.
    建议取值不超过你在 acquire 方法中设置的 resourceExpiredHour 的值.
    计费相关: 页面录制停止前会持续计费, 因此请根据实际业务情况设置合理的值或主动停止页面录制.
    >= 1 <= 720
    """
    maxRecordingHour = None

    """
    type: number

    输出视频的码率, 单位为 Kbps. 针对不同的输出视频分辨率, videoBitrate 的默认值不同:
    输出视频分辨率大于或等于 1280 × 720: 默认值为 2000.
    输出视频分辨率小于 1280 × 720: 默认值为 1500.
    """
    videoBitrate = None

    """
    type: number

    输出视频的帧率, 单位为 fps.
    >= 5 <= 60

    default: 15
    """
    videoFps = None

    """
    type: boolean

    是否开启移动端网页模式:
    true: 开启. 开启后, 录制服务使用移动端网页渲染模式录制当前页面.
    false: (默认)不开启.

    default: false
    """
    mobile = None

    """
    type: number

    页面录制生成的 MP4 切片文件的最大时长, 单位为分钟. 页面录制过程中, 录制服务会在当前 MP4 文件时长超过约 maxVideoDuration 左右时创建一个新的 MP4 切片文件.
    >= 30 <= 240

    default: 120
    """
    maxVideoDuration = None

    """
    type: boolean
    是否在启动页面录制任务时暂停页面录制.
    true: 在启动页面录制任务时暂停页面录制. 开启页面录制任务后立即暂停录制, 录制服务会打开并渲染待录制页面, 但不生成切片文件.
    false: 启动页面录制任务并进行页面录制.

    建议你按照如下流程使用 onhold 字段:
    调用 start 方法时将 onhold 设为 true, 开启并暂停页面录制, 自行判断页面录制开始的合适时机.
    调用 update 并将 onhold 设为 false, 继续进行页面录制. 如果需要连续调用 update 方法暂停或继续页面录制, 请在收到上一次 update 响应后再进行调用, 否则可能导致请求结果与预期不一致.

    default: false
    """
    onhold = None

    """
    type: number

    设置页面加载超时时间, 单位为秒. 详见页面加载超时检测.
    0 或不设置, 表示不检测页面加载状态.
    [1,60] 之间的整数, 表示页面加载超时时间.
    >= 0 <= 60

    default: 0
    """
    readyTimeout = None

class ExtensionServices(request.RequestObject):
    """
    type: required string

    扩展服务的名称:
    web_recorder_service: 代表扩展服务为页面录制.
    rtmp_publish_service: 代表扩展服务为转推页面录制到 CDN.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.ExtensionServiceName`
    """
    serviceName = None

    """
    type: string

    扩展服务内的错误处理策略:
    "error_abort": 页面录制时默认且只能为该值. 表示当前扩展服务出错时, 停止其他扩展服务.
    "error_ignore": 转推页面录制到 CDN 时默认且只能为该值. 表示当前扩展服务出错时, 其他扩展服务不受影响.

    如果页面录制服务或录制上传服务异常, 那么推流到 CDN 失败, 因此页面录制服务出错会影响转推页面录制到 CDN 服务.
    转推到 CDN 的过程发生异常时, 页面录制不受影响.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.ExtensionErrorHandlePolicy`
    """
    errorHandlePolicy = None

    """
    type: required object

    页面录制时需设置如下字段

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.ServiceParam`
    """
    serviceParam = None

class ExtensionServiceConfig(request.RequestObject):
    """
    type: string

    错误处理策略. 默认且仅可设为 "error_abort", 表示当扩展服务发生错误后, 订阅和云端录制的其他非扩展服务都停止.

    default: error_abort

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.ExtensionErrorHandlePolicy`
    """
    errorHandlePolicy = None

    """
    type: required array[object]

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.ExtensionServices`
    """
    extensionServices = None

class AppsCollection(request.RequestObject):
    """
    type: string

    各云端录制应用的组合方式.
    postpone_transcoding: 如需延时转码或延时混音, 则选用此种方式.
    default: 除延时转码和延时混音外, 均选用此种方式.

    default: default

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.CombinationPolicy`
    """
    combinationPolicy = None

class TransConfig(request.RequestObject):
    """
    type: required string

    模式:
    "postponeTranscoding": 延时转码.
    "audioMix": 延时混音.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.PostponeTranscodeMode`
    """
    transMode = None

class Container(request.RequestObject):
    """
    type: string

    文件的容器格式, 支持如下取值:
    "mp4": 延时转码时的默认格式. MP4 格式.
    "mp3": 延时混音时的默认格式. MP3 格式.
    "m4a": M4A 格式.
    "aac": AAC 格式.

    注意: 延时转码暂时只能设为 MP4 格式.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.ContainerFormat`
    """
    format = None

class Audio(request.RequestObject):
    """
    type: string

    音频采样率 (Hz), 支持如下取值:
    "48000": 48 kHz.
    "32000": 32 kHz.
    "16000": 16 kHz.

    default: 48000

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.AudioSampleRate`
    """
    sampleRate = None

    """
    type: string

    音频码率 (Kbps), 支持取值且默认取值为 "48000".

    default: 48000
    """
    bitrate = None

    """
    type: string

    音频声道数, 支持如下取值:
    "1": 单声道.
    "2": 双声道.

    default: 2

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.AudioChannels`
    """
    channels = None

class TranscodeOptions(request.RequestObject):
    """
    type: required object

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.TransConfig`
    """
    transConfig = None

    """
    type: object

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.Container`
    """
    container = None

    """
    type: object

    文件的音频属性.

    注意: 仅需在单流录制模式下, 且开启延时混音时设置.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.Audio`
    """
    audio = Audio()

class ClientRequest(request.RequestObject):
    """
    type: string

    用于鉴权的动态密钥(Token). 如果你的项目已启用 App 证书, 则务必在该字段中传入你项目的动态密钥. 详见使用 Token 鉴权.
    注意:
    仅需在单流录制和合流录制模式下设置.
    云端录制服务暂不支持更新 Token. 为保证录制正常, 请确保 Token 有效时长大于你预计的录制时长, 以避免 Token 过期导致录制任务退出频道而结束录制.
    """
    token = None

    """
    type: required object

    第三方云存储的配置项.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.StorageConfig`
    """
    storageConfig = None

    """
    type: object

    录制的音视频流配置项.
    注意: 仅需在单流录制和合流录制模式下设置.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.RecordingConfig`
    """
    recordingConfig = None

    """
    type: object

    录制文件配置项.
    注意: 仅截图时不可设置该字段, 其他情况都需要设置该字段. 其他情况包含如下:
    单流录制模式下, 不转码录制, 转码录制, 或同时录制和截图.
    合流录制.
    页面录制模式下, 仅页面录制, 或同时页面录制和转推到 CDN.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.RecordingFileConfig`
    """
    recordingFileConfig = None

    """
    type: object

    视频截图配置项.
    注意: 仅需在单流录制模式下, 且使用截图功能时设置.
    截图使用须知:
    截图功能仅适用于单流录制模式(individual).
    你可以在一个单流录制进程中仅截图, 或同时录制和截图, 详见视频截图. 同时录制和截图的情况需要设置 recordingFileConfig 字段.
    如果录制服务或录制上传服务异常, 则截图失败. 截图异常时, 录制不受影响.
    使用截图时, streamTypes 必须设置为 1 或 2. 如果设置了 subscribeAudioUid, 则必须同时设置 subscribeVideoUids.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.SnapshotConfig`
    """
    snapshotConfig = None

    """
    type: object

    扩展服务配置项.
    注意: 仅需在页面录制模式下设置.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.ExtensionServiceConfig`
    """
    extensionServiceConfig = None

    """
    type: object

    应用配置项.
    注意: 仅需在单流录制模式下, 且开启延时转码或延时混音时设置.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.AppsCollection`
    """
    appsCollection = None

    """
    type: object

    延时转码或延时混音下, 生成的录制文件的配置项.
    注意: 仅需在单流录制模式下, 且开启延时转码或延时混音时设置.

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.TranscodeOptions`
    """
    transcodeOptions = None

class RequestPathParamsApiStart(request.RequestObject):
    """
    type: required string

    录制模式:
    individual: 单流录制模式.
    mix: 合流录制模式.
    web: 页面录制模式.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.Mode`
    """
    mode = None

    """
    type: required string

    通过 acquire 请求获取到的 Resource ID.
    """
    resource_id = None

class RequestBodyApiStart(request.RequestObject):
    """
    type: required string

    录制服务所在频道的名称. 需要和你在 acquire 请求中输入的 cname 相同.
    """
    cname = None

    """
    type: required string

    字符串内容为录制服务在 RTC 频道内使用的 UID, 用于标识该录制服务, 需要和你在 acquire 请求中输入的 uid 相同.
    """
    uid = None

    """
    type: required object

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.ClientRequest`
    """
    clientRequest = None

class ResponseApiStart(response.RequestObject):
    """
    type: string

    录制的频道名.
    """
    cname = None

    """
    type: string

    字符串内容为云端录制服务在 RTC 频道内使用的 UID, 用于标识频道内的录制服务.
    """
    uid = None

    """
    type: string

    云端录制资源 Resource ID. 使用这个 Resource ID 可以开始一段云端录制. 这个 Resource ID 的有效期为 5 分钟, 超时需要重新请求.
    """
    resourceId = None

    """
    type: string

    录制 ID. 成功开始云端录制后, 你会得到一个 Sid (录制 ID). 该 ID 是一次录制周期的唯一标识.
    """
    sid = None

    """
    type: number

    状态码
    """
    code = None

def api_start(client, request_path_params_obj, request_body_obj, response_obj=ResponseApiStart, trace_id=None):
    """
    Start recording
    开始云端录制

    :type client: object
    :param client: CloudRecordingClient object

    :type request_path_params_obj: object
    :param request_path_params_obj: request path params object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.RequestPathParamsApiStart`

    :type request_body_obj: object
    :param request_body_obj: request body object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.RequestBodyApiStart`

    :type response_obj: object
    :param response_obj: response object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_start.ResponseApiStart`

    :type trace_id: string
    :param trace_id: trace id

    :return: response object ResponseApiStart
    """
    url = '/v1/apps/{}/cloud_recording/resourceid/{}/mode/{}/start'.format(client.app_id, request_path_params_obj.resource_id, request_path_params_obj.mode)
    client.logger.debug("url:%s", url)

    return client.call_api('POST', url, post_json=request_body_obj.to_dict(), response_obj=response_obj, trace_id=trace_id, is_retry=True)



from agora_rest_client.core import request
from agora_rest_client.services.cloud_recording.v1 import api_update_layout
from agora_rest_client.services.cloud_recording.v1.api import Mode

class LayoutConfig(api_update_layout.LayoutConfig):
    pass

class BackgroundConfig(api_update_layout.BackgroundConfig):
    pass

class ClientRequest(request.RequestObject):
    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.maxResolutionUid`
    """
    maxResolutionUid = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.mixedVideoLayout`
    """
    mixedVideoLayout = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.backgroundColor`
    """
    backgroundColor = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.backgroundImage`
    """
    backgroundImage = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.defaultUserBackgroundImage`
    """
    defaultUserBackgroundImage = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.layoutConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.mix_recording.api_update_layout.LayoutConfig`
    """
    layoutConfig = None

    """
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.backgroundConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.mix_recording.api_update_layout.BackgroundConfig`
    """
    backgroundConfig = None

class RequestPathParamsApiUpdateLayout(api_update_layout.RequestPathParamsApiUpdateLayout):
    pass

class RequestBodyApiUpdateLayout(api_update_layout.RequestBodyApiUpdateLayout):
    pass

class ResponseApiUpdateLayout(api_update_layout.ResponseApiUpdateLayout):
    pass

def mix_recording_update_layout(client, resource_id, sid, cname, uid, max_resolution_uid=None, mixed_video_layout=None, background_color=None,
        background_image=None, default_user_background_image=None, layout_config=None, background_config=None, trace_id=None):
    """
    Mix recording update layout
    更新云端录制合流布局

    :type client: object
    :param client: MixRecordingClient object

    :type resource_id: str
    :param resource_id: resource id
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.RequestPathParamsApiUpdate.resource_id`

    :type sid: str
    :param sid: sid
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.RequestPathParamsApiUpdate.sid`

    :type cname: str
    :param cname: cname
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.RequestBodyApiUpdate.cname`

    :type uid: str
    :param uid: uid
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.RequestBodyApiUpdate.uid`

    :type max_resolution_uid: str
    :param max_resolution_uid: max resolution uid
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.maxResolutionUid`

    :type mixed_video_layout: int
    :param mixed_video_layout: mixed video layout
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.mixedVideoLayout`

    :type background_color: str
    :param background_color: background color
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.backgroundColor`

    :type background_image: str
    :param background_image: background image
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.backgroundImage`

    :type default_user_background_image: str
    :param default_user_background_image: default user background image
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.ClientRequest.defaultUserBackgroundImage`

    :type layout_config: object
    :param layout_config: layout config
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.LayoutConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.mix_recording.api_update_layout.LayoutConfig`

    :type background_config: object
    :param background_config: background config
    :refer: `agora_rest_client.services.cloud_recording.v1.api_update_layout.BackgroundConfig`
    :value: instance of `agora_rest_client.services.cloud_recording.v1.mix_recording.api_update_layout.BackgroundConfig`

    :type trace_id: string
    :param trace_id: trace id

    :return: response object ResponseApiUpdate
    """
    request_path_params_obj = RequestPathParamsApiUpdateLayout(
        mode=Mode.MIX.value,
        resource_id=resource_id,
        sid=sid
    )

    request_body_obj = RequestBodyApiUpdateLayout(
        cname=cname,
        uid=uid,
        clientRequest=ClientRequest()
    )

    if max_resolution_uid is not None:
        request_body_obj.clientRequest.maxResolutionUid = max_resolution_uid

    if mixed_video_layout is not None:
        request_body_obj.clientRequest.mixedVideoLayout = mixed_video_layout

    if background_color is not None:
        request_body_obj.clientRequest.backgroundColor = background_color

    if background_image is not None:
        request_body_obj.clientRequest.backgroundImage = background_image

    if default_user_background_image is not None:
        request_body_obj.clientRequest.defaultUserBackgroundImage = default_user_background_image

    if layout_config is not None:
        request_body_obj.clientRequest.layoutConfig = layout_config

    if background_config is not None:
        request_body_obj.clientRequest.backgroundConfig = background_config

    return api_update_layout.api_update_layout(client, request_path_params_obj=request_path_params_obj, request_body_obj=request_body_obj,
                                               response_obj=ResponseApiUpdateLayout, trace_id=trace_id)



from agora_rest_client.core import request
from agora_rest_client.core import response

"""
官方文档: https://doc.shengwang.cn/api-ref/cloud-recording/restful/cloud-recording/operations/post-v1-apps-appid-cloud_recording-resourceid-resourceid-sid-sid-mode-mode-stop
开始录制后, 你可以调用 stop 方法离开频道, 停止录制. 录制停止后如需再次录制, 必须再调用 acquire 方法请求一个新的 Resource ID.
对于每个声网账号, 每秒钟的请求数限制为 10 次. 如需提高此限制, 请联系技术支持.
stop 请求仅在会话内有效. 如果录制启动错误, 或录制已结束, 调用 stop 将返回 404.
非页面录制模式下, 当频道空闲(无用户)超过预设时间(默认为 30 秒) 后, 云端录制也会自动退出频道, 停止录制.
"""

class ClientRequest(request.RequestObject):
    """
    type: boolean

    设置 stop 方法的响应机制:
    true: 异步. 调用 stop 后立即收到响应.
    false: 同步. 调用 stop 后, 你需等待所有录制文件上传至第三方云存储方后会收到响应. 声网预期上传时间不超过 20 秒, 如果上传超时, 你会收到错误码 50.

    default: false
    """
    async_stop = None

class Payload(response.ResponseObject):
    """
    type: string

    当前录制上传的状态:
    "uploaded": 本次录制的文件已经全部上传至指定的第三方云存储.
    "backuped": 本次录制的文件已经全部上传完成, 但是至少有一个 TS 文件上传到了声网备份云. 声网服务器会自动将这部分文件继续上传至指定的第三方云存储.
    "unknown": 未知状态.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.UploadingStatus`
    """
    uploadingStatus = None

class ExtensionServiceState(response.ResponseObject):
    """
    type: object

    页面录制模式下, 上传服务返回的字段

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_stop.Payload`
    """
    payload = None

    """
    type: string

    服务类型:
    "upload_service": 上传服务.
    "web_recorder_service": 页面录制服务.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.ExtensionServiceStateName`
    """
    serviceName = None

class ServerResponse(response.ResponseObject):
    """
    type: array[object]

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_stop.ExtensionServiceState`
    """
    extensionServiceState = None

class RequestPathParamsApiStop(request.RequestObject):
    """
    type: required string

    录制模式:
    individual: 单流录制模式.
    mix: 合流录制模式.
    web: 页面录制模式.

    :value: enum of `agora_rest_client.services.cloud_recording.v1.api.Mode`
    """
    mode = None

    """
    type: required string

    通过 acquire 请求获取到的 Resource ID.
    """
    resource_id = None

    """
    type: required string

    通过 start 获取的录制 ID.
    """
    sid = None

class RequestBodyApiStop(request.RequestObject):
    """
    type: required string

    录制服务所在频道的名称. 需要和你在 acquire 请求中输入的 cname 相同.
    """
    cname = None

    """
    type: required string

    字符串内容为录制服务在 RTC 频道内使用的 UID, 用于标识该录制服务, 需要和你在 acquire 请求中输入的 uid 相同.
    """
    uid = None

    """
    type: request object

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_stop.ClientRequest`
    """
    clientRequest = None

class ResponseApiStop(response.ResponseObject):
    """
    type: string

    云端录制使用的 Resource ID.
    """
    resourceId = None

    """
    type: string

    录制 ID. 标识一次录制周期.
    """
    sid = None

    """
    type: object

    页面录制场景下会返回的字段

    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_stop.ServerResponse`
    """
    serverResponse = None

    """
    type: string

    录制的频道名.
    """
    cname = None

    """
    type: string

    字符串内容为云端录制服务在 RTC 频道内使用的 UID, 用于标识频道内的录制服务.
    """
    uid = None

def api_stop(client, request_path_params_obj, request_body_obj, response_obj=ResponseApiStop, trace_id=None):
    """
    Stop recording
    停止云端录制

    :type client: object
    :param client: CloudRecordingClient object

    :type request_path_params_obj: object
    :param request_path_params_obj: request path params object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_stop.RequestPathParamsApiStop`

    :type request_body_obj: object
    :param request_body_obj: request body object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_stop.RequestBodyApiStop`

    :type response_obj: object
    :param response_obj: response object
    :value: instance of `agora_rest_client.services.cloud_recording.v1.api_stop.ResponseApiStop`

    :type trace_id: string
    :param trace_id: trace id

    :return: response object ResponseApiStop
    """
    url = '/v1/apps/{}/cloud_recording/resourceid/{}/sid/{}/mode/{}/stop'.format(client.app_id, request_path_params_obj.resource_id, request_path_params_obj.sid, request_path_params_obj.mode)
    client.logger.debug("url:%s", url)

    return client.call_api('POST', url, post_json=request_body_obj.to_dict(), response_obj=response_obj, trace_id=trace_id)



class ClientException(Exception):
    """
    Client exceptions
    """

    def __init__(self, status_code, error_code, error_msg):
        super(ClientException, self).__init__()

        # Http status code
        self.status_code = status_code
        # Error code
        self.error_code = error_code
        # Error message
        self.error_msg = error_msg

    def __str__(self):
        return "%s - {status_code:[%s],error_code:[%s],error_msg:[%s]}" % (self.__class__.__name__, self.status_code, self.error_code, self.error_msg)

class AttributeException(ClientException):
    """
    Attribute exceptions
    """

    def __init__(self, error_msg):
        super(AttributeException, self).__init__(None, None, error_msg)

class ServiceResponseException(ClientException):
    """
    Service response exceptions
    """

    pass

class ClientBuildException(ClientException):
    """
    Client Build Exception
    """

    def __init__(self, error_msg):
        super(ClientBuildException, self).__init__(None, None, error_msg)

class ClientRequestException(ServiceResponseException):
    """
    Client Request Exception
    """

    pass

class ClientTimeoutException(ClientRequestException):
    """
    Client Timeout Exception
    """

    pass
